#include <stdlib.h>
#include <string.h>

#include "draw.h"
#include "level.h"
#include "object.h"

#ifndef NULL
#define	NULL	0
#endif

static Level* level_create(const char* map, int width, int height, const char* tips);
static char readMap(const Level* level, int x, int y);

/* public functions */
Level* level_load(int n)
{
	Level* level = NULL;

	if(n == 1) {
		const char tips[] = "USE THE ARROW KEYS TO MOVE. PICK UP\nTHE EGG TO COMPLETE THE LEVEL.";
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 16, 9, tips);
	} else if(n == 2) {
		const char tips[] = "FLIP THE GRAVITY BY PRESSING SHIFT.\nYOU MUST BE ROTATED THE SAME WAY AS\nTHE EGG TO BE ABLE TO PICK IT UP.";
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 4, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 2, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 16, 11, tips);
	} else if(n == 3) {
		const char tips[] = "USE GRAVITY WELLS TO ROTATE GRAVITY.\nNEW DIRECTION IS BASED ON WHICH WAY\nTHE WELL IS ROTATING.";
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0,12, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 14, 11, tips);
	} else if(n == 4) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 3, 1, 1,
			1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 1, 1, 1, 1, 8, 8, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 7, 7, 1, 1, 1, 1, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 7, 7, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 12, 15, 0);
	} else if(n == 5) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 8, 8, 1, 1, 1, 1, 1, 1, 1, 1, 8, 8, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1,
			1, 1, 2, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 3, 1, 1,
			1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 1, 1, 8, 1, 1, 8, 1, 1, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 7, 7, 1, 1, 1, 7, 7, 1, 1, 1, 7, 7, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 13, 0);
	} else if(n == 6) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 8, 8, 8, 8, 8, 8, 8, 1, 1, 1, 1, 0, 5, 1, 1, 1,
			1, 1, 0,12, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 9, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0, 0, 9, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0, 0, 9, 1, 1,
			1, 1, 0, 0, 0, 1, 1,10, 0, 0, 0, 1, 1, 1, 0, 0, 9, 1, 1,
			1, 1, 0, 0, 0, 1, 1,10, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 1, 2, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 19, 13, 0);
	} else if(n == 7) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 8, 8, 8, 8, 8, 8, 8, 8, 1, 1, 1, 1,
			1, 1, 1,10, 0, 0, 0, 0, 0, 0,12, 0, 9, 1, 1, 1,
			1, 1, 1,10, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1,
			1, 1, 1,10, 0,12, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 0, 0, 2, 0, 0, 1, 1, 0, 0, 0, 0, 3, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 16, 15, 0);
	} else if(n == 8) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 8, 8, 8, 1, 1, 1, 8, 8, 8, 1, 1, 1, 1, 1, 4, 1, 1, 1,
			1, 0,12, 0, 8, 8, 8, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 8, 0, 8, 1, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 9, 1, 1, 8, 0, 0, 0, 8, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 1, 1,10, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 1, 1,10, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0,12, 0, 0, 0, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1,10, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 19, 15, 0);
	} else if(n == 9) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8, 8, 8, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 8, 8, 8, 1, 1, 0, 0, 0, 0, 0, 3, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,13, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0,14, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
			1, 1,13, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 2, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 10) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0,12, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 2, 0, 0, 0, 0, 1, 1, 1, 8, 8, 8, 8, 8, 8, 8, 1, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1,13, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 9, 1, 1, 0, 0, 0, 0, 0, 0, 3, 1, 1,
			1, 1, 1, 1, 1, 0, 0,13, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1,10, 0, 0, 0, 0, 0,14, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 1, 1,10, 0, 0,12, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 1, 1,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 14, 0);
	} else if(n == 11) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 8, 8, 8, 1, 1, 8, 8, 8, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 8, 8, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1,
			1, 1,13, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0,13, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 1, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 7, 7, 1, 1, 7, 7, 1, 1, 7, 7, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 19, 14, 0);
	} else if(n == 12) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 8, 8, 8, 8, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 3, 1,
			1, 1, 1, 1, 7, 7, 1, 1, 0, 0, 0, 0, 1, 1, 7, 7, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 7, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 8, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 7, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 9, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 9, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 13) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1,10, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1,10, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1,10, 0, 0,14,14, 0, 0, 9, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 0,15, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0,15, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0,15, 1, 1,15, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 3, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 14, 0);
	} else if(n == 14) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 8, 8, 8, 1, 1, 1, 0, 0,16, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 8, 8, 8, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 7, 7, 7, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0,12, 0, 1, 1, 1, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 4, 0, 9, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0,13, 1, 1, 1, 1, 0, 0, 9, 1, 1,
			1, 1, 2, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,11, 9, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1,16, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 15) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 6, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0,14, 0, 0, 1,
			1, 0,12, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 7, 7, 0, 0, 7, 0, 0, 0,14, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 9, 1, 1,10, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 9, 1, 1,10, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 8, 8, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 16) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 1,
			1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 8, 8, 8, 8, 1, 1, 1, 1, 1,
			1,13, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
			1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0,16, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 1, 1, 0,12, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 1, 1,13, 0, 0, 1, 1, 1,16, 0, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 19, 15, 0);
	} else if(n == 17) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 8, 8, 8, 8, 8, 8, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10,12, 0, 0, 0, 0, 5, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 0, 7, 7, 7, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 0, 0,12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,11, 9, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 2, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 18) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 8, 8, 1, 1, 1, 1, 8, 8, 1, 1, 1, 1, 8, 8, 1,
			1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1, 3, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,15, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 7, 7, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,17, 0, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,19, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 2, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 14, 0);
	} else if(n == 19) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 8, 8, 8, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0,12, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 1, 1, 8, 8, 8, 8, 8, 8, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 8, 8, 1, 1, 1, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8, 1, 1,
			1,18, 0, 0, 0, 1, 1, 1, 0, 0, 0, 7, 7, 7, 0, 0, 0, 0, 8, 1,
			1, 1, 0, 0, 0,19, 1, 1,13, 0, 0, 1, 1, 1, 7, 7, 0,12, 0, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1,
			1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0,17, 1, 1, 1, 1, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 3, 0, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 20) {
		const char map[] = {
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 8, 8, 1, 1,
			1, 0, 0, 0, 0, 0,12, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 3, 0, 0, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8, 8, 8, 0, 0,11, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 7, 7, 7, 1, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 14, 0);
	} else if(n == 21) {
		const char map[] = {
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 4, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0,15, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0,15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0,15, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 22) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0,17, 1, 1, 0, 0, 1,
			1, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1, 0, 0, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 1, 1, 0, 0,13, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 0,13, 0, 0, 0, 0, 1, 1, 0,11, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1,13, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 0, 0, 0,11, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 4, 9, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0,19, 9, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 14, 0);
	} else if(n == 23) {
		const char map[] = {
			1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 1, 1, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0,21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,22,
			1, 2, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0,11, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 24) {
		const char map[] = {
			1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0,21, 0, 0, 0, 0,22, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1,
			1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1,
			1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1,13, 0, 0, 1, 1, 1, 1, 7, 7, 7, 7, 1, 1, 1, 1, 0, 0,13, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 25) {
		const char map[] = {
			0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0,
			0, 0, 0, 0, 0, 0,14, 0, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,14, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 8, 8, 8, 8, 0, 0, 8, 8, 8, 8, 0, 0, 0, 0, 0,
			0, 0, 0, 0,21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,22, 0, 0, 0, 0,
			0,12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,11, 0,
			0, 0, 0, 0, 0, 1, 0, 7, 7, 7, 7, 7, 7, 0, 1, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0,
			0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0,
			0, 1, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
			0, 0, 0, 0, 0, 0,12, 0, 0, 1, 1, 0, 0,11, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 26) {
		const char map[] = {
			1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 0, 0, 0,12, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 8, 8, 0, 0, 0, 0, 0,
			0, 0, 0,11, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			7, 7, 7, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 0, 0, 7,
			1, 1, 1, 1, 1, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 1, 1, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 4, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0,21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 0, 0, 1, 1, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 27) {
		const char map[] = {
			1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 0, 0,12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
			1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 0,19, 1, 1,10, 0, 0, 0, 0, 0, 1, 0,
			1, 0, 0, 0, 0, 0, 0, 0,17, 1, 1, 1,10, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,21, 0,
			1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 0, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 0, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 2, 1, 1, 1, 1,10, 0, 0, 0, 0, 0, 1, 1,
			1, 7, 7, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 0, 0, 5, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,10, 0, 0, 0, 0, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 28) {
		const char map[] = {
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 4, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,21, 0, 0, 0, 1, 1, 0, 0, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0,24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0,
			0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,24, 0, 0, 0, 0, 0,
			1, 2, 0, 0, 0, 0, 0, 0,11, 0, 0, 0, 0, 0, 0, 0, 0, 0,11, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 29) {
		const char map[] = {
			0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0,
			0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0,
			0, 1, 1, 8, 8, 8, 8, 8, 0, 0, 0, 0, 0, 8, 0, 0, 0,11, 0, 0,
			0, 1, 1, 6, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 1, 1, 7, 7, 7, 7, 7, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			0, 1, 1,26, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0,24, 0, 0,28, 0, 0, 0, 0, 0,
			0, 1, 1, 2, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 30) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 2, 0, 0, 0, 1, 1, 0, 0,15, 0, 0, 0, 0,12, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1,26, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 1,
			1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
			1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 1,
			1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 1,
			1, 0,25, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,15, 0, 0, 0, 0, 5, 1,
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 31) {
		const char map[] = {
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 1,
			0, 0, 1, 1, 0, 0, 1,10, 0, 0, 0, 0, 7, 7, 7, 0, 0, 0, 0, 1,
			0, 0, 1, 1,26, 0, 1,10, 0, 0, 0, 9, 1, 1, 1,10, 0, 0, 0, 0,
			0, 0, 1, 1, 0, 0, 1,10, 0, 0, 0, 9, 1, 1, 1,10, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 1,10, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8, 8, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 0, 0, 1, 1, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 0,25, 1, 1, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1, 0, 0, 1, 1, 0, 0,
			1, 2, 0, 0, 0, 0, 0,11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 32) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1,
			1, 0,12, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0,12, 0, 1,
			1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1,
			1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 1,
			1, 0,25, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,26, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 7, 7, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 8, 8, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,10, 0, 9, 1,
			1, 1,30, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1,10, 0, 9, 1,
			1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1,10, 0, 9, 1,
			1, 2, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1,10, 3, 9, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 33) {
		const char map[] = {
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 1,
			1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1,
			1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0,19, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 0, 0, 0, 1, 1,18, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 8, 8, 8, 8, 8,
			1, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0,11, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 7, 0, 0, 7, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 9, 1,10, 0, 9, 1, 0, 9, 1,10,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 8, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0,11, 0, 0, 0, 0, 9, 1,10, 0, 0, 0, 0,11, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 34) {
		const char map[] = {
			1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0,
			1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1,
			1, 0, 0, 0, 0, 1, 0, 0,21, 0, 0,22, 0, 0, 1, 0, 0, 1, 1, 1,
			1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 7, 7, 7, 7, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 35) {
		const char map[] = {
			 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0,
			 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			20, 0, 0, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 7, 7, 0, 0, 0, 0,17,
			 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1,
			 1, 0, 0, 0, 0, 1, 1, 8, 8, 8, 8, 8, 8, 1, 1, 0, 0, 0, 0, 1,
			 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1,
			 1, 1, 1, 0, 0, 1, 1, 0,11, 0, 0,11, 0, 1, 1, 0, 0, 1, 1, 1,
			 1, 1, 1,26, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0,25, 1, 1, 1,
			 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			 0, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 36) {
		const char map[] = {
			1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			1, 0, 0, 0,18, 0, 0, 0, 0,23, 0, 0, 0,27, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
			1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0,
			1, 3, 0, 1, 1, 1, 1, 1, 0,21, 0,22, 0, 1, 1, 1, 1, 0, 0, 0,
			1, 1, 1, 1, 1, 8, 8, 8, 0, 0, 0, 0, 0, 8, 8, 8, 8, 0, 0, 0,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 37) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 1, 1, 1,13, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 2, 0, 0, 1, 1, 1, 0, 0,12, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0,13, 1, 1, 1, 1, 1, 1, 1, 1,
			1,13, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0,24, 0, 0, 9, 1,
			1, 1, 0,25, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1,
			1, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 9, 1,
			0, 0, 0, 0, 1, 1, 1, 0, 0, 7, 7, 7, 1, 1, 1, 1, 1, 0, 9, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 9, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 9, 1,
			0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 9, 1,
			0,11, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 3, 0, 0,27, 0, 0, 9, 1,
			0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 38) {
		const char map[] = {
			1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1,
			1, 2, 0, 1, 1, 0, 0, 0,17, 1, 1,18, 0, 0, 0, 1, 1, 0, 3, 1,
			1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 0,25, 1, 1, 1, 1, 1, 1,26, 0, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 8, 8, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0,29, 1, 1, 1, 0, 0, 1, 1, 1,30, 0, 1, 1, 1, 1,
			1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1,
			1,10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1,
			1,10,11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,11, 9, 1,
			1,10, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 9, 1,
			1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 39) {
		const char map[] = {
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 4, 1,
			0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0,21, 0, 0, 0, 0, 0, 0,22, 0, 0, 1, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
			0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 2, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0,21, 0, 0, 0, 1, 0, 0,
			1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	} else if(n == 40) {
		const char map[] = {
			0, 0, 0, 0, 1, 1,10, 0, 0, 0,21, 0, 0, 0, 1, 1, 1, 1, 1, 1,
			0,12, 0, 0, 1, 1,10, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 4, 9, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 1,
			1, 1, 0,25, 1, 1, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1,26, 0, 1, 1,
			1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 0, 0, 1, 1,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 1,26, 0, 1, 1,10, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0,25, 1, 1,
			1, 1, 0, 0, 1, 1,10, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1,
			1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
			1, 2, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 0, 0,11, 0,
			1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 9, 1, 1, 0, 0, 0, 0
		};
		level = level_create(map, 20, 15, 0);
	}
	if(level) level->id = n;
	return level;
}

void level_free(Level* level)
{
	int i;
	if(level) {
		if(level->object) {
			for(i=0 ; level->object[i] ; i++)
				free(level->object[i]);
			free(level->object);
		}
		if(level->tips) free(level->tips);
		if(level->map) free(level->map);
		free(level);
	}
}

/* private functions */
static Level* level_create(const char* map, int width, int height, const char* tips)
{
	Level* level;
	Object* object;
	int x, y;
	
	level = malloc(sizeof(Level));
	if(!level) return NULL;

	level->width = width;
	level->height = height;
	level->gravity = DOWN;
	level->object = malloc(sizeof(Object*));
	if(!level->object) {
		level_free(level);
		return NULL;
	}
	level->object[0] = 0;

	level->map = malloc(width*height);
	if(!level->map) {
		level_free(level);
		return NULL;
	}

	if(tips) {
		level->tips = malloc(strlen(tips)+1);
		if(!level->tips) {
			level_free(level);
			return NULL;
		}
		memcpy(level->tips, tips, strlen(tips)+1);
	} else level->tips = NULL;

	for(y=0 ; y<height ; y++) {
		for(x=0 ; x<width ; x++) {
			switch(map[y*width+x]) {
				case BEGIN:
					level->object = object_add(level->object, object_new(DUCK, x*TILE_WIDTH, y*TILE_HEIGHT, UP, RIGHT));
					level->map[y*width+x] = EMPTY;
					break;
				case EGG_UP:
					level->object = object_add(level->object, object_new(EGG, x*TILE_WIDTH, y*TILE_HEIGHT, UP, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case EGG_DOWN:
					level->object = object_add(level->object, object_new(EGG, x*TILE_WIDTH, y*TILE_HEIGHT, DOWN, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case EGG_LEFT:
					level->object = object_add(level->object, object_new(EGG, x*TILE_WIDTH, y*TILE_HEIGHT, LEFT, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case EGG_RIGHT:
					level->object = object_add(level->object, object_new(EGG, x*TILE_WIDTH, y*TILE_HEIGHT, RIGHT, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case GRAVITY_PLUS:
					object = object_new(GRAVITY_SWITCH, x*TILE_WIDTH, y*TILE_HEIGHT, UP, UP);
					object->var = 1;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case GRAVITY_MINUS:
					object = object_new(GRAVITY_SWITCH, x*TILE_WIDTH, y*TILE_HEIGHT, UP, UP);
					object->var = -1;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case HORIZONTAL_CUBE:
					object = object_new(CUBE, x*TILE_WIDTH, y*TILE_HEIGHT, UP, UP);
					object->dx = 1;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case VERTICAL_CUBE:
					object = object_new(CUBE, x*TILE_WIDTH, y*TILE_HEIGHT, UP, UP);
					object->dy = 1;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case HORIZONTAL_CATERPILLAR:
					object = object_new(CATERPILLAR, x*TILE_WIDTH, y*TILE_HEIGHT, map[(y+1)*width+x]==GROUND? UP:DOWN, RIGHT);
					object->dx = 1;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case VERTICAL_CATERPILLAR:
					object = object_new(CATERPILLAR, x*TILE_WIDTH, y*TILE_HEIGHT, map[y*width+x+1]==GROUND? LEFT:RIGHT, DOWN);
					object->dy = 1;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case PLANT_LEFT_1:
					level->object = object_add(level->object, object_new(SHOOTING_PLANT, x*TILE_WIDTH, y*TILE_HEIGHT, UP, LEFT));
					level->map[y*width+x] = EMPTY;
					break;
				case PLANT_RIGHT_1:
					level->object = object_add(level->object, object_new(SHOOTING_PLANT, x*TILE_WIDTH, y*TILE_HEIGHT, UP, RIGHT));
					level->map[y*width+x] = EMPTY;
					break;
				case PLANT_LEFT_2:
					object = object_new(SHOOTING_PLANT, x*TILE_WIDTH, y*TILE_HEIGHT, UP, LEFT);
					object->var = TIME_PLANT/2;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case PLANT_RIGHT_2:
					object = object_new(SHOOTING_PLANT, x*TILE_WIDTH, y*TILE_HEIGHT, UP, RIGHT);
					object->var = TIME_PLANT/2;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case BEE_1:
					level->object = object_add(level->object, object_new(BEE, x*TILE_WIDTH, y*TILE_HEIGHT, UP, RIGHT));
					level->map[y*width+x] = EMPTY;
					break;
				case BEE_2:
					object = object_new(BEE, x*TILE_WIDTH, y*TILE_HEIGHT, UP, RIGHT),
					object->var = TIME_BEE/2;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_UP_1:
					level->object = object_add(level->object, object_new(PESTLE, x*TILE_WIDTH, (y-1)*TILE_HEIGHT, UP, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_DOWN_1:
					level->object = object_add(level->object, object_new(PESTLE, x*TILE_WIDTH, y*TILE_HEIGHT, DOWN, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_LEFT_1:
					level->object = object_add(level->object, object_new(PESTLE, (x-1)*TILE_WIDTH, y*TILE_HEIGHT, LEFT, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_RIGHT_1:
					level->object = object_add(level->object, object_new(PESTLE, x*TILE_WIDTH, y*TILE_HEIGHT, RIGHT, UP));
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_UP_2:
					object = object_new(PESTLE, x*TILE_WIDTH, (y-1)*TILE_HEIGHT, UP, UP);
					object->var = TIME_PESTLE/2;
					object->position = 5;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_DOWN_2:
					object = object_new(PESTLE, x*TILE_WIDTH, y*TILE_HEIGHT, DOWN, UP);
					object->var = TIME_PESTLE/2;
					object->position = 5;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_LEFT_2:
					object = object_new(PESTLE, (x-1)*TILE_WIDTH, y*TILE_HEIGHT, LEFT, UP);
					object->var = TIME_PESTLE/2;
					object->position = 5;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				case PESTLE_RIGHT_2:
					object = object_new(PESTLE, x*TILE_WIDTH, y*TILE_HEIGHT, RIGHT, UP);
					object->var = TIME_PESTLE/2;
					object->position = 5;
					level->object = object_add(level->object, object);
					level->map[y*width+x] = EMPTY;
					break;
				default: level->map[y*width+x] = map[y*width+x];
			}
		}
	}
	// Choose bitmap for each GROUND box
	object = object_get(level->object, EGG);
	for(y=0 ; y<height ; y++) {
		for(x=0 ; x<width ; x++) {
			if(map[y*width+x] == GROUND) {
				int near, far, diag;
				Ground_tile tab[] = {G_UDLR, G_GRASS_UDL, G_GRASS_UDR, G_GRASS_UD, G_GRASS_ULR, G_GRASS_UL, G_GRASS_UR, G_GRASS_U, G_DLR, G_DL, G_DR, G_D, G_LR, G_L, G_R};
				Ground_tile tab_angle[] = {0, G_ANGLE_UL, G_ANGLE_UR, 0, G_ANGLE_DL, 0, 0, 0, G_ANGLE_DR};
				near = readMap(level, x, y-1)<<3 | readMap(level, x, y+1)<<2 | readMap(level, x-1, y)<<1 | readMap(level, x+1, y);
				far = readMap(level, x, y-2) & readMap(level, x, y+2) & readMap(level, x-2, y) & readMap(level, x+2, y) & readMap(level, x-2, y-2) & readMap(level, x+2, y-2) & readMap(level, x-2, y+2) & readMap(level, x+2, y+2);
				diag = (readMap(level, x+1, y+1)<<3 | readMap(level, x-1, y+1)<<2 | readMap(level, x+1, y-1)<<1 | readMap(level, x-1, y-1))^15;
				if(object->head==UP && object->x/TILE_WIDTH==x && object->y/TILE_WIDTH==y-1) level->map[y*width+x] = G_EGG_UP;
				else if(object->head==DOWN && object->x/TILE_WIDTH==x && object->y/TILE_WIDTH==y+1) level->map[y*width+x] = G_EGG_DOWN;
				else if(object->head==LEFT && object->x/TILE_WIDTH==x-1 && object->y/TILE_WIDTH==y) level->map[y*width+x] = G_EGG_LEFT;
				else if(object->head==RIGHT && object->x/TILE_WIDTH==x+1 && object->y/TILE_WIDTH==y) level->map[y*width+x] = G_EGG_RIGHT;
				else if(near < 15) level->map[y*width+x] = tab[near];
				else if(diag && diag < 9) level->map[y*width+x] = tab_angle[diag];
				else if(far) level->map[y*width+x] = G_OUT;
				else level->map[y*width+x] = G_NORMAL;
			}
		}
	}
	return level;
}

static char readMap(const Level* level, int x, int y)
{
	if(x<0 || x>=level->width || y<0 || y>=level->height) return 1;
	return (level->map[y*level->width+x]==GROUND || level->map[y*level->width+x]>=G_OUT);
}
