#include "graphic_functions.h"

#include "draw.h"
#include "level.h"
#include "object.h"
#include "tileset.h"

/* global variables */
Tileset *ground, *spike[4], *egg[4], *egg_caught, *duck[4][4], *gravity_switch, *cube, *caterpillar[4][4], *plant[4], *bullet, *bee[4], *bee_halo, *pestle[4], *blast[2];

/* public functions */
int draw_init()
{
	ground = tileset_load(T_GROUND);

	spike[UP] = tileset_load(T_SPIKE_UP);
	spike[DOWN] = tileset_load(T_SPIKE_DOWN);
	spike[LEFT] = tileset_load(T_SPIKE_LEFT);
	spike[RIGHT] = tileset_load(T_SPIKE_RIGHT);

	egg[UP] = tileset_load(T_EGG_UP);
	egg[DOWN] = tileset_load(T_EGG_DOWN);
	egg[LEFT] = tileset_load(T_EGG_LEFT);
	egg[RIGHT] = tileset_load(T_EGG_RIGHT);
	egg_caught = tileset_load(EGG_CAUGHT);

	duck[UP][RIGHT] = tileset_load(DUCK_UP_RIGHT);
	duck[UP][LEFT] = tileset_load(DUCK_UP_LEFT);
	duck[DOWN][RIGHT] = tileset_load(DUCK_DOWN_RIGHT);
	duck[DOWN][LEFT] = tileset_load(DUCK_DOWN_LEFT);
	duck[RIGHT][UP] = tileset_load(DUCK_RIGHT_UP);
	duck[RIGHT][DOWN] = tileset_load(DUCK_RIGHT_DOWN);
	duck[LEFT][UP] = tileset_load(DUCK_LEFT_UP);
	duck[LEFT][DOWN] = tileset_load(DUCK_LEFT_DOWN);

	gravity_switch = tileset_load(T_GRAVITY_SWITCH);
	cube = tileset_load(T_CUBE);

	caterpillar[UP][RIGHT] = tileset_load(CATERPILLAR_UP_RIGHT);
	caterpillar[UP][LEFT] = tileset_load(CATERPILLAR_UP_LEFT);
	caterpillar[DOWN][RIGHT] = tileset_load(CATERPILLAR_DOWN_RIGHT);
	caterpillar[DOWN][LEFT] = tileset_load(CATERPILLAR_DOWN_LEFT);
	caterpillar[RIGHT][UP] = tileset_load(CATERPILLAR_RIGHT_UP);
	caterpillar[RIGHT][DOWN] = tileset_load(CATERPILLAR_RIGHT_DOWN);
	caterpillar[LEFT][UP] = tileset_load(CATERPILLAR_LEFT_UP);
	caterpillar[LEFT][DOWN] = tileset_load(CATERPILLAR_LEFT_DOWN);

	plant[LEFT] = tileset_load(T_PLANT_LEFT);
	plant[RIGHT] = tileset_load(T_PLANT_RIGHT);

	bullet = tileset_load(T_BULLET);

	bee[LEFT] = tileset_load(T_BEE_LEFT);
	bee[RIGHT] = tileset_load(T_BEE_RIGHT);
	bee_halo = tileset_load(BEE_HALO);

	pestle[UP] = tileset_load(T_PESTLE_UP);
	pestle[DOWN] = tileset_load(T_PESTLE_DOWN);
	pestle[LEFT] = tileset_load(T_PESTLE_LEFT);
	pestle[RIGHT] = tileset_load(T_PESTLE_RIGHT);

	blast[0] = tileset_load(BIG_BLAST);
	blast[1] = tileset_load(SMALL_BLAST);

	// if all tilesets are correctly loaded
	if(	ground &&
		spike[UP] && spike[DOWN] && spike[UP] && spike[LEFT] && spike[RIGHT] &&
		egg[UP] && egg[DOWN] && egg[LEFT] && egg[RIGHT] &&
		duck[UP][RIGHT] && duck[UP][LEFT] && duck[DOWN][RIGHT] && duck[DOWN][LEFT] && duck[RIGHT][UP] && duck[RIGHT][DOWN] && duck[LEFT][UP] && duck[LEFT][DOWN] &&
		gravity_switch && cube &&
		caterpillar[UP][RIGHT] && caterpillar[UP][LEFT] && caterpillar[DOWN][RIGHT] && caterpillar[DOWN][LEFT] && caterpillar[RIGHT][UP] && caterpillar[RIGHT][DOWN] && caterpillar[LEFT][UP] && caterpillar[LEFT][DOWN] &&
		plant[LEFT] && plant[RIGHT] &&
		bullet &&
		bee[LEFT] && bee[RIGHT] && bee_halo &&
		pestle[UP] && pestle[DOWN] && pestle[LEFT] && pestle[RIGHT] &&
		blast[0] && blast[1])
		return 0;
	// else
	draw_quit();
	return 1;
}

void draw_quit()
{
	tileset_free(ground);

	tileset_free(spike[UP]);
	tileset_free(spike[DOWN]);
	tileset_free(spike[LEFT]);
	tileset_free(spike[RIGHT]);

	tileset_free(egg[UP]);
	tileset_free(egg[DOWN]);
	tileset_free(egg[LEFT]);
	tileset_free(egg[RIGHT]);
	tileset_free(egg_caught);

	tileset_free(duck[UP][RIGHT]);
	tileset_free(duck[UP][LEFT]);
	tileset_free(duck[DOWN][RIGHT]);
	tileset_free(duck[DOWN][LEFT]);
	tileset_free(duck[RIGHT][UP]);
	tileset_free(duck[RIGHT][DOWN]);
	tileset_free(duck[LEFT][UP]);
	tileset_free(duck[LEFT][DOWN]);

	tileset_free(gravity_switch);
	tileset_free(cube);

	tileset_free(caterpillar[UP][RIGHT]);
	tileset_free(caterpillar[UP][LEFT]);
	tileset_free(caterpillar[DOWN][RIGHT]);
	tileset_free(caterpillar[DOWN][LEFT]);
	tileset_free(caterpillar[RIGHT][UP]);
	tileset_free(caterpillar[RIGHT][DOWN]);
	tileset_free(caterpillar[LEFT][UP]);
	tileset_free(caterpillar[LEFT][DOWN]);

	tileset_free(plant[LEFT]);
	tileset_free(plant[RIGHT]);

	tileset_free(bullet);

	tileset_free(bee[LEFT]);
	tileset_free(bee[RIGHT]);
	tileset_free(bee_halo);

	tileset_free(pestle[UP]);
	tileset_free(pestle[DOWN]);
	tileset_free(pestle[LEFT]);
	tileset_free(pestle[RIGHT]);

	tileset_free(blast[0]);
	tileset_free(blast[1]);
}

void draw_tileMap(Level* level, Camera camera)
{
	int i, j, x, y, n;
	for(j=0 ; j<SCREEN_HEIGHT/TILE_HEIGHT+2 ; j++) {
		y = TILE_HEIGHT*j - (camera.y&(TILE_HEIGHT-1));
		for(i=0 ; i<SCREEN_WIDTH/TILE_WIDTH+1 ; i++) {
			x = TILE_WIDTH*i - (camera.x&(TILE_WIDTH-1));
			if((camera.x>>4)+i>=0 && (camera.y>>4)+j>=0 && (camera.x>>4)+i<level->width && (camera.y>>4)+j<level->height) {
				switch(n = level->map[((camera.y>>4)+j)*level->width + (camera.x>>4)+i]) {
					case SPIKE_UP: CopySpriteClippingMasked(spike[UP]->tile[0], spike[UP]->palette, x, y, TILE_WIDTH, TILE_HEIGHT, MASK); break;
					case SPIKE_DOWN: CopySpriteClippingMasked(spike[DOWN]->tile[0], spike[DOWN]->palette, x, y, TILE_WIDTH, TILE_HEIGHT, MASK); break;
					case SPIKE_LEFT: CopySpriteClippingMasked(spike[LEFT]->tile[0], spike[LEFT]->palette, x, y, TILE_WIDTH, TILE_HEIGHT, MASK); break;
					case SPIKE_RIGHT: CopySpriteClippingMasked(spike[RIGHT]->tile[0], spike[RIGHT]->palette, x, y, TILE_WIDTH, TILE_HEIGHT, MASK); break;
					default: if(n>=G_OUT) CopySpriteClippingMasked(ground->tile[n-G_OUT], ground->palette, x, y, TILE_WIDTH, TILE_HEIGHT, MASK);
				}
			} else CopySpriteClippingMasked(ground->tile[0], ground->palette, x, y, TILE_WIDTH, TILE_HEIGHT, MASK);
		}
	}
}

void draw_objects(Level* level, Camera camera)
{
	int i;
	Object* object;
	for(i=0 ; level->object[i] ; i++) {
		object = level->object[i];
		switch(object->type) {
			case DUCK: CopySpriteMasked(duck[object->head][object->look]->tile[(int)object->position], duck[object->head][object->look]->palette, object->x-camera.x, object->y-camera.y, duck[object->head][object->look]->width, duck[object->head][object->look]->height, MASK); break;
			case EGG:
				if(object->position >= egg[object->head]->length) {
					CopySpriteMasked(egg[object->head]->tile[13], egg[object->head]->palette, object->x-camera.x, object->y-camera.y, egg[object->head]->width, egg[object->head]->height, MASK);
					CopySpriteMasked(egg_caught->tile[object->position-egg[object->head]->length], egg_caught->palette, object->x-camera.x, object->y-camera.y, egg_caught->width, egg_caught->height, MASK);
				} else CopySpriteMasked(egg[object->head]->tile[(int)object->position], egg[object->head]->palette, object->x-camera.x, object->y-camera.y, egg[object->head]->width, egg[object->head]->height, MASK);
				break;
			case GRAVITY_SWITCH: CopySpriteMasked(gravity_switch->tile[(int)object->position], gravity_switch->palette, object->x-camera.x, object->y-camera.y, gravity_switch->width, gravity_switch->height, MASK); break;
			case CUBE: CopySpriteMasked(cube->tile[0], cube->palette, object->x-camera.x, object->y-camera.y, cube->width, cube->height, MASK); break;
			case CATERPILLAR: CopySpriteMasked(caterpillar[object->head][object->look]->tile[(int)object->position], caterpillar[object->head][object->look]->palette, object->x-camera.x, object->y-camera.y, caterpillar[object->head][object->look]->width, caterpillar[object->head][object->look]->height, MASK); break;
			case SHOOTING_PLANT: CopySpriteMasked(plant[object->look]->tile[(int)object->position], plant[object->look]->palette, object->x-camera.x, object->y-camera.y, plant[object->look]->width, plant[object->look]->height, MASK); break;
			case BEE:
				CopySpriteMasked(bee[object->look]->tile[(int)object->position], bee[object->look]->palette, object->x-camera.x, object->y-camera.y, bee[object->look]->width, bee[object->look]->height, MASK);
				if(object->var < 6) CopySpriteMasked(bee_halo->tile[object->var/2+3], bee_halo->palette, object->x-camera.x, object->y-camera.y, bee_halo->width, bee_halo->height, MASK);
				if(object->var >= TIME_BEE-2-3*3) CopySpriteMasked(bee_halo->tile[(object->var-(TIME_BEE-2-3*3))%3], bee_halo->palette, object->x-camera.x, object->y-camera.y, bee_halo->width, bee_halo->height, MASK);
				break;
			case BULLET: CopySpriteMasked(bullet->tile[0], bullet->palette, object->x-camera.x, object->y-camera.y, bullet->width, bullet->height, MASK); break;
			case HOMING_BULLET: CopySpriteMasked(bullet->tile[1], bullet->palette, object->x-camera.x, object->y-camera.y, bullet->width, bullet->height, MASK); break;
			case PESTLE: CopySpriteMasked(pestle[object->head]->tile[(int)object->position], pestle[object->head]->palette, object->x-camera.x, object->y-camera.y, pestle[object->head]->width, pestle[object->head]->height, MASK); break;
			case DUCK_BLAST: CopySpriteClippingMaskedAlpha(blast[0]->tile[0], blast[0]->palette, object->x-(blast[0]->width-duck[UP][LEFT]->width)/2-camera.x, object->y-(blast[0]->height-duck[UP][LEFT]->height)/2-camera.y, blast[0]->width, blast[0]->height, MASK, 32-object->var); break;
			case BULLET_BLAST: CopySpriteClippingMaskedAlpha(blast[1]->tile[0], blast[1]->palette, object->x-(blast[1]->width-bullet->width)/2-camera.x, object->y-(blast[1]->height-bullet->height)/2-camera.y, blast[1]->width, blast[1]->height, MASK, 32-object->var); break;
		}
	}
}

void draw_background()
{
	char compressed_bitmap[] = {
		0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,2,3,4,1,2,3,3,3,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,1,2,3,3,1,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,1,3,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,2,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,1,1,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,3,4,1,1,1,1,1,2,4,1,1,2,3,3,3,3,3,3,3,3,4,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,4,1,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,2,1,1,1,1,1,1,3,3,4,2,3,4,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,4,1,1,3,4,1,1,2,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,2,3,4,1,3,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,3,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,1,1,1,2,1,1,1,2,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,3,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,1,1,1,3,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,2,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,4,1,2,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,4,1,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,4,1,1,1,4,1,1,1,1,1,1,1,1,1,2,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,2,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,1,1,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,4,2,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,4,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,2,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,4,1,2,3,1,2,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,1,1,1,1,4,1,1,1,1,1,1,2,4,1,1,
		1,1,1,1,1,1,1,1,1,2,1,1,1,1,3,3,3,4,2,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,1,1,1,3,3,1,1,1,1,1,1,3,4,1,1,
		1,2,4,1,1,1,1,1,1,3,3,1,1,2,3,3,3,4,2,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,1,1,2,3,3,1,1,2,1,1,2,3,3,4,1,
		1,3,3,1,1,3,1,1,1,3,3,4,1,2,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,1,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,1,1,3,3,4,2,3,3,3,3,
		3,3,3,1,1,3,4,1,1,3,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,1,1,3,3,3,3,3,3,3,3,
		3,3,3,4,2,3,3,1,2,3,3,3,3,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,2,3,3,4,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,4,2,3,3,3,3,3,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,6,3,3,3,5,3,3,3,3,3,3,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,6,3,5,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,4,1,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,3,3,5,7,3,3,3,3,3,3,4,2,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,3,3,3,3,3,5,7,6,7,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,2,1,1,1,1,1,1,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,3,3,3,5,3,3,4,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,4,1,3,3,3,3,3,3,3,7,7,7,7,3,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,3,1,1,2,1,1,1,3,3,4,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,3,3,5,7,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,6,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,1,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,3,4,1,3,4,1,1,3,3,3,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,3,3,7,7,3,3,3,3,3,1,1,1,1,1,4,1,1,1,1,1,1,1,1,1,3,4,1,1,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,3,3,3,3,3,1,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,4,1,1,2,3,3,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,3,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,3,3,3,3,3,3,4,1,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,1,1,1,1,1,3,1,1,1,1,3,1,1,1,1,3,3,4,1,3,3,3,3,3,3,3,5,3,3,7,7,7,7,7,7,7,7,3,3,3,3,3,2,3,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,3,4,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,6,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,3,3,3,3,3,3,3,2,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,1,1,4,1,2,3,4,1,1,1,3,3,1,1,1,3,3,3,3,3,3,3,3,3,3,3,5,6,3,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,1,1,1,1,1,1,1,1,1,1,1,2,3,4,1,1,2,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,6,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,6,3,5,7,7,3,3,3,3,3,3,3,3,3,3,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,3,3,3,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,4,2,3,1,2,3,3,1,1,2,3,3,3,1,1,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,1,1,2,1,1,1,2,4,1,1,1,2,3,3,1,1,2,3,3,1,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,6,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,3,5,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,6,3,5,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,1,3,3,3,3,4,2,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,1,1,3,1,1,1,3,3,4,1,1,2,3,3,1,1,3,3,3,4,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,3,7,7,6,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,6,3,5,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,6,3,3,7,7,7,6,7,7,7,7,7,7,7,7,7,7,7,3,3,5,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,6,3,5,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,4,2,3,4,1,2,3,3,3,4,1,2,3,3,4,1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,3,3,7,7,7,7,7,6,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,6,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,4,2,3,3,3,3,4,2,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,3,3,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,3,5,3,
		3,3,3,3,3,3,3,3,3,3,3,3,3,6,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,7,7,3,
		3,3,3,3,3,3,3,3,3,5,6,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,9,7,7,7,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,3,
		3,3,6,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,9,9,7,7,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,7,7,7,7,7,9,9,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,6,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,3,
		3,5,7,3,3,5,6,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,9,9,10,8,9,10,7,7,10,7,7,7,7,7,7,7,7,7,7,7,3,5,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,3,3,5,7,7,7,8,7,7,9,9,10,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,6,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,3,
		6,7,7,3,3,7,7,3,3,7,7,7,7,7,7,8,10,7,7,7,7,10,7,7,7,7,7,7,7,7,7,3,3,3,3,7,3,3,3,3,3,3,3,7,7,7,7,7,7,8,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,3,7,7,7,7,8,9,8,9,9,10,7,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,
		7,7,10,6,3,7,7,7,7,7,7,10,7,7,8,9,10,7,7,7,9,9,7,7,10,7,7,7,7,7,7,6,3,3,5,7,6,3,3,3,3,3,3,7,7,7,7,7,7,8,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,3,6,3,3,3,6,3,3,3,5,7,7,7,7,7,7,8,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,6,3,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,
		7,8,10,7,8,10,7,7,7,7,9,10,7,7,9,9,9,7,7,7,9,9,9,9,10,7,7,7,7,7,7,7,3,3,7,7,7,3,3,3,7,7,7,7,7,7,7,7,7,9,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,5,7,3,3,5,7,3,3,3,5,7,7,7,7,7,7,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,6,5,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,6,3,3,3,3,3,3,3,3,3,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		9,9,9,8,9,10,7,7,7,9,9,9,7,9,9,9,9,10,7,9,9,9,9,9,10,7,7,7,7,7,7,7,6,5,7,7,7,6,3,3,7,7,7,7,7,7,7,10,7,9,9,9,9,9,9,9,9,9,10,7,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,7,7,6,3,5,7,6,3,3,7,7,7,7,7,7,7,9,9,9,9,9,10,7,7,9,10,7,7,7,7,7,7,7,7,7,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,5,7,3,3,3,3,3,3,3,5,7,3,3,7,7,7,7,7,7,7,7,7,7,10,7,7,7,7,7,7,7,7,
		9,9,9,9,9,9,10,7,8,9,9,9,8,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,6,5,7,7,7,7,7,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,7,7,7,7,8,7,7,9,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,6,3,3,3,3,3,3,3,3,3,3,3,3,3,3,7,7,6,3,5,6,3,3,3,7,7,7,5,7,7,7,7,7,7,7,7,7,8,10,7,7,7,7,7,7,7,7,
		9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,7,7,7,7,7,8,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,6,7,6,3,3,6,3,3,6,3,3,5,7,7,7,7,7,3,5,7,6,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,8,10,7,7,7,7,7,7,7,7,
		9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,10,7,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,8,10,7,7,7,10,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,5,7,3,5,7,3,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,10,7,7,7,9,9,7,9,10,7,7,7,7,7,
		9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,9,7,7,9,9,9,11,12,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,9,9,7,7,7,10,7,7,7,7,7,7,7,7,7,7,7,7,7,8,7,7,7,7,7,7,7,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,10,7,7,7,8,10,7,7,8,10,7,7,7,9,9,9,9,7,7,7,7,7,7,
		9,11,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,7,7,9,7,7,7,7,8,10,7,8,9,10,7,9,9,9,13,13,9,9,9,9,9,9,9,9,9,9,9,9,10,7,9,9,9,7,7,8,9,7,7,7,7,8,7,7,7,8,7,7,7,7,9,10,7,7,7,10,7,7,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,9,7,7,7,9,9,7,7,8,9,7,7,8,9,9,9,9,7,7,7,7,7,7,
		12,13,9,9,9,12,9,9,9,9,9,9,9,9,11,9,9,9,9,13,9,9,9,9,9,9,9,9,7,7,7,8,9,10,7,7,7,9,10,7,8,9,9,9,9,9,9,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,8,9,10,7,7,7,9,10,7,7,9,10,7,7,7,9,9,7,7,8,9,7,8,9,9,9,9,9,9,9,9,9,9,9,9,7,8,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,10,7,7,8,9,10,7,7,9,9,10,7,9,9,10,7,9,9,9,9,9,7,7,7,7,7,7,
		13,13,12,9,13,13,9,9,9,9,9,9,9,11,13,12,9,9,11,13,12,9,9,9,9,9,9,9,7,7,7,9,9,10,7,7,7,9,9,7,8,9,9,9,9,9,9,13,13,13,9,9,11,13,13,9,9,9,9,9,9,9,9,9,9,9,9,7,9,9,9,9,7,8,9,9,7,7,9,9,10,7,8,9,9,10,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,9,10,7,8,9,10,7,8,9,9,10,7,9,9,9,9,9,9,9,9,9,7,8,10,7,7,7,
		13,13,13,11,13,13,9,9,13,13,12,9,9,13,13,12,9,13,13,13,13,13,13,9,9,9,9,9,10,7,8,9,9,9,7,7,8,9,9,9,9,9,9,9,9,9,9,13,13,13,12,13,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,8,9,9,9,10,8,9,9,10,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,9,9,9,9,9,9,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,7,
		13,13,13,13,13,13,12,13,13,13,13,9,11,13,13,13,13,13,13,13,13,13,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,11,12,9,9,13,13,13,13,13,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,7,
		13,13,13,13,13,13,13,13,13,13,13,12,13,13,13,13,13,13,13,13,13,13,13,9,9,11,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,13,13,9,11,13,13,13,13,13,13,13,13,13,9,9,9,9,9,9,9,9,13,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,10,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,8,7,
		13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,11,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,13,13,9,13,13,13,13,13,13,13,13,13,13,9,9,11,9,9,9,9,11,13,13,12,9,9,13,9,9,9,9,9,9,11,12,9,9,9,9,9,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,
		14,13,13,13,13,13,13,15,14,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,11,13,13,9,9,9,9,12,9,9,9,9,9,9,9,9,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,13,12,9,9,9,13,13,13,12,9,13,13,9,9,9,9,9,9,13,13,9,9,9,9,11,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,9,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,8,10,7,7,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,
		14,13,14,13,13,13,15,15,15,13,13,13,13,13,15,13,13,13,13,16,13,13,13,13,12,13,13,13,9,9,9,11,13,9,9,9,11,13,12,9,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,12,13,13,13,9,9,11,13,13,13,12,11,13,13,12,9,9,9,11,13,13,13,12,9,9,11,13,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,7,8,10,7,7,7,8,9,7,7,7,7,8,9,7,7,7,9,9,9,9,9,9,9,9,9,11,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,
		15,17,15,13,13,17,15,15,15,15,13,13,17,15,15,14,13,18,19,16,13,13,13,13,13,13,13,13,12,9,11,13,13,9,9,9,13,13,13,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,11,13,13,13,13,13,13,13,13,9,9,11,13,13,13,13,13,9,9,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,7,7,8,9,9,7,7,7,8,9,9,7,7,7,9,9,7,7,8,9,9,9,9,9,9,9,9,11,13,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,7,
		15,15,15,14,13,15,15,15,15,15,15,13,15,15,15,20,19,19,19,16,13,13,13,13,13,13,13,13,12,9,13,13,13,9,9,11,13,13,13,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,13,13,13,13,13,13,13,13,13,12,9,13,13,13,13,13,13,9,13,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,7,7,9,9,9,10,7,7,9,9,9,10,7,8,9,9,10,7,8,9,9,9,9,9,9,9,9,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,10,
		15,15,15,15,17,15,15,15,15,15,15,15,15,15,15,20,19,19,19,16,13,13,21,13,13,13,13,13,13,11,13,13,13,12,11,13,13,13,13,12,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,17,14,13,13,13,13,13,13,9,13,13,13,13,13,13,13,13,13,13,13,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,11,13,13,12,9,9,9,9,9,10,7,8,9,9,9,10,7,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,13,12,9,9,9,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,19,19,16,22,23,21,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,15,13,13,13,13,13,13,13,13,13,13,13,15,15,15,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,11,9,9,9,9,9,9,13,9,9,9,9,11,13,13,13,12,9,9,9,9,9,7,8,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,13,13,9,9,11,13,13,13,13,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,19,16,24,25,21,26,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,17,15,13,13,13,13,15,15,13,13,13,15,15,15,13,13,13,13,15,15,15,15,27,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,13,9,9,9,9,11,13,13,13,9,9,11,13,13,13,13,12,9,9,9,9,9,10,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,11,13,13,13,9,13,13,13,13,13,9,9,11,13,13,9,9,9,9,9,9,9,9,9,9,11,13,9,9,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,19,19,24,25,21,26,13,17,15,14,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,15,15,13,13,13,15,15,15,14,13,17,15,15,15,15,13,13,13,15,15,15,20,16,22,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,13,12,9,9,9,13,13,13,13,12,11,13,13,13,13,13,13,9,9,9,11,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,11,9,9,9,9,13,13,13,13,13,13,13,13,13,13,9,9,13,13,13,12,9,9,9,9,9,13,9,9,9,13,13,12,9,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,19,19,24,25,21,26,13,15,15,15,13,13,13,13,17,14,13,13,13,13,13,13,13,13,13,17,15,15,14,13,15,15,15,15,14,13,15,15,15,15,15,14,13,17,15,15,15,20,16,25,13,13,13,13,13,15,15,13,13,13,13,13,13,13,13,13,13,13,9,11,13,12,9,9,9,13,13,13,13,13,13,13,13,13,13,13,13,9,9,11,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,13,13,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,9,11,13,13,9,9,11,13,13,12,9,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,19,19,24,25,21,26,15,15,15,15,13,13,13,13,15,15,13,13,13,28,24,13,13,13,13,17,15,15,14,13,15,15,15,15,15,17,15,15,15,15,15,15,14,17,15,15,15,19,16,25,21,15,14,13,13,15,15,14,13,13,13,13,13,13,13,13,13,13,12,11,13,13,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,9,9,9,9,9,9,9,9,9,9,9,9,11,13,13,12,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,9,13,13,13,12,9,13,13,13,13,9,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,19,19,24,24,21,21,15,15,15,15,14,13,13,15,15,15,27,13,13,28,24,22,13,13,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,24,25,21,15,15,13,17,15,15,15,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,13,9,9,9,9,9,9,9,9,9,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,13,13,13,13,13,13,13,13,13,12,
		15,15,15,15,15,15,15,15,15,15,29,15,15,15,15,15,15,19,19,19,16,24,21,21,15,15,15,15,14,13,15,15,15,15,19,13,13,28,24,22,13,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,24,21,30,15,15,15,15,15,15,15,14,13,17,15,27,13,13,13,26,13,13,13,13,13,9,13,13,13,13,13,17,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,9,13,12,9,9,9,9,9,9,9,9,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,11,13,13,13,13,13,13,13,13,13,13,
		15,15,15,15,15,15,15,15,15,31,29,15,15,15,15,15,31,19,19,19,16,24,21,21,15,15,15,15,15,15,15,15,15,15,19,27,13,24,24,21,13,23,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,16,24,21,30,15,15,15,15,15,15,15,15,15,15,15,20,27,13,24,26,13,13,13,13,13,13,13,13,13,13,13,17,15,13,13,13,13,13,13,13,13,13,13,13,13,9,9,11,13,13,9,9,9,9,9,9,13,12,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,27,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
		15,15,15,15,15,15,15,15,15,31,29,15,15,15,15,15,32,19,19,19,16,24,21,21,15,15,15,15,15,15,15,15,15,20,19,19,13,24,24,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,16,24,21,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,24,22,13,13,13,13,13,13,13,13,13,13,13,17,15,13,13,13,17,14,13,13,13,13,13,13,13,12,9,13,13,13,12,9,9,9,9,13,13,13,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,20,19,16,13,13,13,13,13,13,13,13,13,13,13,14,13,13,13,
		15,15,15,15,15,15,15,15,15,32,29,15,15,15,15,15,32,33,19,19,19,24,25,21,15,15,15,15,15,15,15,15,15,20,19,16,24,24,24,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,24,25,21,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,16,24,13,13,13,13,13,13,13,13,13,13,13,15,15,15,13,13,17,15,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,9,13,13,13,12,11,13,13,13,13,13,13,13,13,13,13,13,15,14,13,13,13,17,20,19,16,24,22,13,13,13,13,13,13,13,13,17,20,16,13,13,
		29,15,15,15,15,15,15,15,31,32,29,15,15,15,15,15,32,33,19,19,19,24,25,21,30,15,15,15,15,15,15,15,15,20,19,16,24,24,25,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,24,25,21,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,19,24,13,13,13,13,13,13,13,13,13,13,13,15,15,15,15,15,15,15,14,13,13,15,15,13,13,13,13,13,13,13,13,9,9,9,11,13,13,13,12,11,13,17,15,13,13,13,13,17,13,13,17,15,15,14,13,17,15,20,19,16,24,24,26,13,13,13,13,13,13,13,15,20,24,21,15,
		32,15,15,15,15,15,15,15,32,32,15,15,15,15,15,31,32,33,19,19,19,24,25,21,30,15,15,15,15,15,15,15,15,19,19,24,24,24,25,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,33,16,24,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,16,22,13,13,13,13,13,13,13,13,13,17,15,15,15,15,15,15,15,14,13,17,15,15,13,13,13,13,13,13,13,13,9,9,9,11,13,13,13,13,11,13,15,15,14,13,13,17,15,15,17,15,15,15,15,13,17,15,15,19,16,24,24,26,13,13,13,13,13,13,13,15,20,24,21,15,
		32,15,15,15,32,32,32,32,32,32,15,15,15,15,15,31,32,32,19,19,19,16,24,21,34,15,15,15,15,15,15,15,15,19,19,24,24,24,21,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,19,16,24,21,34,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,16,22,13,13,13,17,13,13,13,13,13,17,15,15,15,15,15,15,15,15,13,17,15,15,14,13,13,13,13,13,13,13,12,9,9,13,13,13,13,13,13,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,16,24,24,21,13,13,17,15,13,13,13,15,20,25,21,15,
		32,29,15,15,32,32,32,32,32,32,15,15,15,15,15,32,32,32,19,19,19,16,24,21,21,15,15,15,15,15,15,15,15,19,19,24,24,24,21,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,15,33,19,24,25,21,32,32,15,15,15,15,15,32,15,15,15,15,15,15,15,20,19,19,22,13,13,13,15,14,13,13,13,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,13,13,13,13,14,13,13,13,9,9,13,13,13,13,13,13,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,16,24,24,21,26,17,15,15,15,15,15,15,19,25,21,15,
		32,32,15,15,31,32,32,32,32,32,15,15,15,15,31,32,32,32,35,19,19,19,24,21,21,15,31,15,15,15,15,15,20,19,16,24,24,25,21,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,33,24,24,25,21,32,32,29,15,15,15,31,32,15,15,15,31,15,15,15,15,19,19,22,13,23,15,15,14,13,17,13,17,15,15,15,15,15,15,15,15,15,15,15,15,15,15,13,13,13,15,15,27,13,13,12,9,13,13,13,17,14,13,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,16,24,24,21,30,15,15,15,15,15,15,15,16,25,30,15,
		32,32,15,15,31,32,32,32,32,32,15,15,15,15,31,32,32,36,35,19,19,19,24,25,21,15,31,32,32,32,29,15,20,19,16,24,24,25,21,21,30,15,15,31,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,19,24,24,21,34,32,32,29,15,15,31,32,32,15,15,15,31,29,15,15,15,20,19,24,25,21,30,15,15,14,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,13,13,17,15,15,19,13,13,13,13,13,21,15,15,15,14,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,24,24,25,21,15,15,15,15,15,15,20,16,25,34,32,
		32,32,15,15,31,32,32,32,32,32,15,15,15,15,31,37,36,36,35,19,19,19,24,25,21,30,31,32,32,32,29,15,20,19,16,24,24,25,21,21,30,15,15,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,33,16,24,25,21,32,32,32,32,15,15,32,32,32,15,15,15,31,32,15,15,15,15,19,16,24,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,13,13,15,15,20,19,27,13,13,13,13,30,15,15,15,15,13,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,24,24,25,21,15,15,15,15,15,15,20,24,21,34,32,
		32,32,15,15,31,32,32,32,32,29,15,15,15,15,32,36,36,36,36,19,19,19,16,25,21,30,31,32,32,32,29,15,20,19,24,24,24,21,21,21,30,15,31,32,32,29,15,15,15,29,15,15,15,15,15,15,15,15,15,15,15,19,24,24,21,21,32,32,32,32,29,32,32,32,29,15,15,15,31,32,29,15,15,32,19,19,24,25,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,14,15,15,15,20,19,19,22,13,13,23,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,16,24,24,21,15,15,15,15,15,15,19,24,21,34,32,
		32,32,15,15,31,32,32,32,32,29,15,15,15,15,36,36,36,36,36,19,19,19,16,24,21,21,31,32,32,32,29,15,19,19,24,24,24,21,21,21,15,15,31,32,32,32,15,15,31,29,15,15,15,31,32,15,15,15,15,15,20,16,24,25,21,34,32,32,32,32,32,32,32,32,29,15,15,15,32,32,32,32,32,32,19,19,16,24,21,21,15,15,31,29,15,15,15,15,15,31,32,29,15,32,32,29,15,15,15,15,15,15,15,20,19,19,24,13,13,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,24,24,21,30,15,15,15,15,15,16,24,21,34,32,
		32,32,15,15,31,32,32,32,32,29,15,15,15,38,36,36,36,36,36,35,19,19,19,24,21,21,31,32,32,32,29,15,19,19,24,24,25,21,21,21,32,32,32,32,32,32,29,15,31,32,32,32,32,32,32,15,15,15,15,15,19,24,24,21,21,32,32,32,32,32,32,32,32,32,29,15,15,15,32,32,32,32,32,32,33,19,19,24,25,21,30,31,32,29,15,15,15,15,15,32,32,32,32,32,32,29,15,15,15,15,15,15,15,19,19,16,24,22,13,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,24,24,25,21,15,15,15,15,15,16,25,21,34,32,
		32,32,15,15,31,32,32,32,32,29,15,15,15,38,36,36,36,36,36,35,19,19,19,24,21,21,34,32,32,32,29,15,19,16,24,24,25,21,21,21,32,32,32,32,32,32,32,29,32,32,32,32,32,32,32,29,15,15,15,20,16,24,25,21,30,32,32,32,32,32,32,32,32,32,29,15,15,15,32,32,32,32,32,32,29,19,19,16,24,21,34,32,32,29,15,15,15,31,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,19,19,16,24,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,16,24,25,21,30,15,15,15,20,16,25,21,34,32,
		32,32,15,15,15,32,32,32,32,29,15,15,15,36,36,36,36,36,36,32,19,19,19,24,25,21,34,32,32,32,29,15,19,16,24,24,21,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,20,19,24,24,21,21,31,32,32,32,32,36,39,32,32,32,29,15,15,15,32,32,32,32,32,32,15,19,19,16,24,25,21,32,32,32,15,15,15,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,20,19,19,24,24,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,29,15,15,15,15,15,15,15,15,19,19,24,24,21,30,15,15,15,20,24,21,21,32,32,
		39,32,15,15,15,36,32,32,32,29,15,15,15,36,36,36,36,36,36,32,19,19,19,16,25,21,21,32,32,32,29,40,19,16,24,24,21,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,19,24,24,25,21,30,31,32,36,39,32,36,39,32,32,32,29,15,15,31,32,32,32,32,32,32,15,20,19,19,24,25,21,32,32,32,15,15,31,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,19,19,16,24,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,15,15,15,15,15,15,15,15,19,19,24,24,25,21,15,15,15,19,24,21,21,32,32,
		39,32,15,15,15,36,36,36,32,15,15,15,15,36,36,36,36,36,39,32,33,19,19,19,24,21,21,34,32,32,29,20,19,16,24,25,21,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,19,16,24,25,21,21,29,31,32,36,39,32,36,36,32,37,32,15,15,15,31,32,32,32,32,32,29,15,15,19,19,16,24,21,34,32,32,29,15,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,19,19,24,25,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,29,31,15,15,15,15,15,15,20,19,16,24,25,21,30,15,20,16,25,21,21,32,32,
		39,32,15,15,15,36,36,36,39,15,15,15,15,36,36,36,36,36,39,32,33,19,19,19,24,25,21,21,32,32,32,20,19,24,24,25,21,21,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,16,24,24,21,21,34,15,31,37,36,36,36,36,36,36,36,32,15,15,15,31,32,32,32,32,29,15,15,31,19,19,16,24,25,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,15,19,19,16,25,21,21,15,15,15,15,32,29,15,15,15,15,15,15,15,15,15,15,15,31,32,32,29,31,15,15,15,15,15,15,15,19,19,24,24,21,21,15,20,24,25,21,34,32,32,
		36,32,15,15,15,36,36,36,36,15,15,15,15,36,36,36,36,36,39,32,32,19,19,19,16,25,21,21,34,32,32,20,19,24,24,21,21,21,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,24,24,21,21,30,31,15,31,37,36,36,36,36,36,36,36,29,15,15,15,32,32,32,32,32,29,15,15,31,33,19,19,24,25,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,15,19,19,24,25,21,30,15,15,15,31,32,32,29,15,15,15,15,15,15,15,15,15,15,31,32,32,29,15,31,15,15,15,15,15,15,20,19,16,24,25,21,30,19,24,25,21,34,32,32,
		36,32,15,15,15,36,36,36,36,15,15,15,15,36,36,36,36,36,32,32,32,19,19,19,16,24,21,21,34,32,32,19,19,24,24,21,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,33,19,16,24,24,25,21,21,21,15,31,15,31,36,36,36,36,36,36,36,36,29,15,15,15,32,32,32,32,29,15,15,15,32,32,19,19,24,25,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,20,19,19,24,21,30,15,15,15,15,32,32,32,32,32,15,15,15,15,32,15,15,15,15,31,32,32,29,15,32,32,29,15,15,15,15,15,19,19,24,24,21,21,16,24,21,21,32,32,32,
		36,37,41,15,15,38,36,36,36,15,15,15,15,36,36,36,36,36,32,32,32,33,19,19,19,24,25,21,21,32,29,19,19,24,24,21,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,24,24,24,21,21,21,15,15,15,15,31,36,36,36,36,36,36,36,36,29,15,15,15,32,32,32,15,15,15,15,31,32,32,19,19,24,24,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,20,19,16,25,21,15,15,15,15,31,32,32,32,32,32,15,15,15,15,32,29,15,15,15,31,32,32,29,15,32,32,15,15,15,15,15,15,20,19,16,24,25,21,24,24,21,21,32,32,32,
		36,36,41,15,15,38,36,36,36,15,15,15,15,36,36,36,36,36,32,32,32,33,19,19,19,24,24,21,21,34,29,19,16,24,25,21,21,21,34,32,32,32,32,32,32,32,32,32,32,33,19,19,24,24,24,24,21,21,34,15,15,15,15,15,32,36,36,36,36,36,36,36,36,15,15,15,31,32,32,15,15,15,15,31,32,32,32,33,19,16,24,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,31,15,19,19,24,21,34,29,15,15,15,31,32,32,32,32,29,15,15,15,31,32,32,32,15,15,31,32,32,29,31,32,29,15,15,15,15,15,15,15,19,19,24,24,21,42,24,21,21,32,32,32,
		36,36,36,15,15,15,36,36,36,15,15,15,15,36,36,36,36,36,32,39,32,32,19,19,19,16,24,25,21,34,20,19,24,24,25,21,21,21,34,32,32,32,32,32,32,32,32,32,33,19,16,24,24,24,25,21,21,21,32,15,15,15,15,15,32,36,36,36,36,36,36,36,41,15,15,15,31,32,15,15,15,15,15,32,32,32,32,33,19,16,24,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,32,15,19,19,24,21,34,29,15,15,15,31,32,32,32,32,29,15,29,15,32,32,32,32,29,15,31,32,32,15,31,32,15,15,15,15,15,15,15,15,33,19,16,24,25,21,24,21,21,32,32,32,
		36,36,36,15,15,15,36,36,36,15,15,15,38,36,36,36,36,36,37,39,32,32,19,19,19,16,24,25,21,21,20,19,24,24,25,21,21,21,32,32,32,32,32,32,32,32,32,32,19,16,24,24,24,25,21,21,34,32,32,15,15,15,15,15,32,37,36,36,36,36,36,36,41,15,15,15,31,29,15,15,15,15,32,32,32,32,32,32,19,19,24,25,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,31,32,33,19,16,24,21,32,32,15,15,15,31,32,32,32,32,29,15,29,31,32,32,32,32,29,15,15,32,32,15,31,15,15,15,15,29,15,15,15,15,32,19,19,24,25,21,42,21,21,32,32,32,
		36,36,36,41,15,15,38,36,36,15,15,15,38,36,36,36,36,36,36,36,32,32,33,19,19,19,24,24,21,21,43,19,24,24,25,21,21,21,32,32,32,32,32,32,32,32,32,33,19,24,24,24,25,21,21,32,32,32,32,29,15,15,15,15,32,37,36,36,36,36,36,36,15,15,15,15,29,15,15,15,15,32,32,32,32,32,32,32,19,19,24,25,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,33,19,16,25,21,32,32,15,15,15,31,32,32,32,32,29,31,29,32,32,32,32,32,29,15,15,31,32,15,15,15,15,15,15,32,41,15,15,31,32,33,19,16,24,25,21,21,21,32,32,32,
		36,36,36,36,15,15,15,36,36,15,15,15,38,36,36,36,36,36,36,36,32,32,33,19,19,19,16,24,25,21,43,19,24,24,25,21,21,21,32,32,32,32,32,32,32,32,32,19,24,24,21,21,21,21,34,32,32,32,32,32,15,15,15,15,32,37,36,36,36,36,36,41,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,33,19,24,25,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,16,25,34,32,32,15,15,15,31,32,32,32,32,15,31,32,32,32,32,32,32,29,15,15,31,32,15,15,15,15,15,31,37,36,15,15,32,32,32,19,16,24,25,21,21,21,32,32,32,
		36,36,36,36,41,15,15,36,36,15,15,15,38,36,36,36,36,36,36,36,32,32,33,19,19,19,16,24,25,21,43,19,24,24,25,21,21,21,32,32,32,32,32,32,32,32,33,16,24,21,21,21,21,34,32,32,32,32,32,32,15,15,15,15,32,37,36,36,36,36,41,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,33,19,16,25,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,21,34,32,32,15,15,15,15,32,32,32,32,15,32,32,32,32,32,32,32,29,15,15,31,32,15,15,15,15,15,32,32,36,41,15,32,32,32,33,19,24,24,21,21,21,32,32,32,
		36,36,36,36,41,15,15,38,41,15,15,15,38,36,36,36,36,36,36,36,32,39,32,19,19,19,19,24,24,21,21,19,24,24,25,21,21,21,32,32,32,32,32,32,32,32,16,24,24,21,21,21,32,32,32,32,32,32,32,32,15,15,15,15,32,37,36,36,36,36,41,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,33,19,16,25,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,21,34,32,32,15,15,15,15,32,32,32,29,31,32,32,32,32,32,32,32,32,15,15,31,29,15,15,15,15,36,32,32,37,36,36,32,32,32,35,19,16,24,25,21,21,44,39,32,
		36,36,36,36,36,15,15,15,41,15,15,15,38,36,36,36,36,36,36,36,36,39,32,19,19,19,19,24,24,21,21,19,24,24,25,21,21,21,32,32,32,32,32,32,32,19,16,24,25,21,21,34,32,32,32,32,32,32,32,32,29,15,15,15,32,37,36,36,36,36,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,33,19,16,24,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,33,19,16,25,21,32,32,32,15,15,15,15,32,32,32,29,31,32,32,32,32,32,32,32,32,15,15,15,15,15,15,38,36,36,39,32,37,36,36,39,32,32,36,19,16,24,24,21,21,44,36,32,
		36,36,36,36,36,41,15,15,41,15,15,15,38,36,36,36,36,36,36,36,36,36,32,33,19,19,19,16,24,25,21,43,24,24,25,21,21,21,32,32,32,32,32,32,33,19,24,24,21,21,34,32,32,32,32,32,32,32,32,32,29,15,15,15,36,36,36,36,36,41,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,33,19,16,25,21,32,32,32,15,15,15,15,32,32,32,15,31,32,32,32,32,32,32,32,32,15,15,15,15,15,15,38,36,36,39,32,32,36,36,39,32,37,36,19,16,24,24,21,21,44,36,39,
		36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,32,33,19,19,19,16,24,25,21,43,24,24,25,21,21,21,32,32,32,32,32,32,19,16,24,24,21,21,32,32,32,32,32,32,32,32,32,32,32,15,15,15,36,36,36,36,36,41,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,21,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,25,34,32,32,32,29,15,15,15,32,32,32,15,32,32,32,32,32,32,32,32,32,29,15,15,15,15,31,36,36,36,36,32,32,36,36,39,32,37,36,35,19,24,24,21,21,21,36,39,
		36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,39,33,19,19,19,16,24,24,21,43,24,24,25,21,21,21,32,32,32,32,32,33,16,24,24,25,21,34,32,32,32,32,32,32,32,32,32,32,32,15,15,15,38,36,36,36,36,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,25,34,32,32,32,29,15,15,15,32,32,29,15,32,32,32,32,32,32,32,32,32,15,15,15,15,15,32,36,36,36,36,32,32,36,36,32,32,37,36,36,19,24,24,25,21,21,36,36,
		36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,39,37,19,19,19,19,24,24,21,43,24,24,25,21,21,21,32,32,32,32,32,19,16,24,24,21,21,34,32,32,32,32,32,32,32,32,32,32,32,15,15,15,38,36,36,36,36,15,15,15,15,15,38,32,32,32,32,32,32,36,36,32,32,32,32,32,32,33,19,24,25,34,32,32,32,32,32,32,32,32,32,32,32,32,32,33,19,16,24,21,34,32,32,32,29,15,15,15,32,32,29,15,32,32,32,32,32,32,32,32,32,15,15,15,15,15,37,36,36,36,36,39,32,37,36,32,32,36,36,36,19,16,24,25,21,21,44,36,
		36,36,36,36,36,36,36,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,39,37,19,19,19,19,24,24,21,42,24,24,21,21,21,21,32,32,32,32,33,16,24,24,21,21,21,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,38,36,36,36,41,15,15,15,15,15,36,32,32,37,36,36,36,36,36,39,32,32,32,32,32,33,19,24,25,21,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,16,24,21,34,32,32,32,29,15,15,15,32,32,29,31,32,32,32,32,32,32,32,32,32,15,15,15,15,31,37,36,36,36,36,39,32,37,39,32,32,36,36,36,35,16,24,24,21,21,44,36,
		36,36,36,36,36,36,36,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,37,19,19,19,19,24,24,21,42,24,24,21,21,21,21,32,32,32,32,19,24,24,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,37,41,15,15,38,36,36,36,15,15,15,15,15,15,36,39,37,36,36,36,36,36,36,36,32,32,32,32,32,33,19,24,25,21,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,24,25,21,32,32,32,32,29,15,15,15,32,32,15,31,32,32,32,32,32,32,32,32,32,15,15,15,15,31,36,36,36,36,36,36,32,32,39,32,37,36,36,36,35,19,24,24,21,21,21,36,
		36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,19,19,19,19,24,24,21,42,24,24,21,21,21,21,32,32,32,19,19,24,24,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,37,41,15,15,15,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,33,19,24,24,21,32,32,32,32,32,32,32,32,32,32,32,32,33,19,19,24,25,21,32,32,32,32,29,15,15,15,32,32,15,32,32,32,32,32,32,32,32,32,32,15,15,15,15,32,36,36,36,36,36,36,32,32,32,32,37,36,36,36,36,19,24,24,25,21,21,36,
		36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,21,42,24,24,21,21,21,21,32,32,33,19,24,24,24,21,21,21,32,32,32,32,32,32,32,32,32,32,32,32,37,41,15,15,15,36,36,41,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,32,32,32,32,32,19,24,24,21,32,32,32,32,32,32,32,32,32,32,32,32,33,19,16,24,25,34,32,32,32,32,29,15,15,15,32,29,15,32,32,32,32,32,32,32,32,32,29,15,15,15,15,32,36,36,36,36,36,36,39,32,32,32,37,36,36,36,36,19,16,24,25,21,21,44,
		36,36,36,36,36,36,36,36,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,21,42,24,24,21,21,21,21,32,19,19,16,24,24,24,21,21,21,32,32,32,32,32,32,32,32,36,36,36,39,37,41,15,15,15,36,36,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,39,32,37,36,36,19,24,24,21,34,32,32,32,32,32,32,32,32,32,32,32,19,19,24,24,21,34,32,32,32,32,29,15,15,15,32,29,31,32,32,32,32,32,32,32,32,32,15,15,15,15,15,32,36,36,36,36,36,36,39,32,32,32,37,36,36,36,36,35,16,24,24,21,21,44,
		36,36,36,36,36,36,36,36,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,21,42,24,24,21,21,21,21,33,19,19,24,24,24,25,21,21,34,32,32,32,32,32,32,32,37,36,36,36,36,36,41,15,15,15,36,41,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,32,37,36,36,19,16,24,25,34,32,32,32,32,32,32,32,32,32,32,33,19,19,24,24,21,34,32,32,37,36,41,15,15,15,32,15,38,36,32,32,32,32,32,32,32,32,15,15,15,15,31,32,37,36,36,36,36,36,36,32,32,32,36,36,36,36,36,35,19,24,24,21,21,21,
		36,36,36,36,36,36,36,36,41,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,21,42,24,24,21,21,21,21,43,19,19,24,24,24,25,21,21,32,32,32,37,36,39,32,32,36,36,36,36,36,36,41,15,15,15,38,41,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,39,36,36,36,19,16,24,25,34,32,32,32,32,32,32,32,32,32,32,33,19,16,24,25,21,32,32,32,36,36,41,15,15,15,29,15,38,36,39,32,32,37,36,36,39,29,15,15,15,15,31,32,37,36,36,36,36,36,36,32,32,32,36,36,36,36,36,36,19,24,24,25,21,21,
		36,36,36,36,36,36,36,36,41,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,19,19,19,24,24,21,42,24,24,21,21,21,21,43,19,16,24,24,24,21,21,21,39,32,32,36,36,36,32,32,36,36,36,36,36,36,41,15,15,15,38,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,16,24,25,21,32,32,32,32,32,37,36,36,36,33,19,19,24,24,25,34,32,32,37,36,36,41,15,15,15,15,15,36,36,36,32,32,36,36,36,36,29,15,15,15,15,31,32,32,36,36,36,36,36,36,32,32,32,36,36,36,36,36,36,19,16,24,25,21,21,
		36,36,36,36,36,36,36,36,41,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,41,19,19,19,24,24,21,42,24,24,21,21,21,21,19,19,16,24,24,24,21,21,21,32,32,37,36,36,36,32,32,36,36,36,36,36,36,36,15,15,15,38,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,16,24,25,21,32,32,32,32,32,37,36,36,35,19,19,19,24,24,21,34,32,32,36,36,36,41,15,15,15,15,15,36,36,36,32,32,36,36,36,36,41,15,15,15,15,38,32,32,37,36,36,36,36,36,39,32,32,36,36,36,36,36,36,35,16,24,24,21,21,
		36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,41,19,19,19,24,24,21,42,24,24,21,21,21,21,19,19,24,24,24,25,21,21,44,32,32,37,36,36,36,39,32,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,32,32,32,32,32,37,36,35,19,19,19,16,24,25,21,44,32,32,36,36,36,41,15,15,15,15,15,15,36,36,39,37,36,36,36,36,15,15,15,15,15,38,32,32,32,36,36,36,36,36,39,32,32,36,36,36,36,36,36,35,19,24,24,21,21,
		36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,15,19,19,19,24,24,21,42,24,24,21,21,21,21,19,19,24,24,24,25,21,21,34,32,32,36,36,36,36,39,32,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,39,32,32,36,32,37,36,19,19,19,19,24,24,25,21,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,36,36,36,36,36,36,15,15,15,15,15,38,39,32,32,37,36,36,36,36,39,32,32,36,36,36,36,36,36,36,19,24,24,25,21,
		36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,15,19,19,19,24,24,21,42,24,24,21,21,21,21,43,16,24,24,24,25,21,21,34,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,44,32,32,37,39,37,35,19,19,19,16,24,24,21,44,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,36,36,36,36,36,36,15,15,15,15,15,36,36,32,32,32,36,36,36,36,39,32,32,36,36,36,36,36,37,36,19,16,24,25,21,
		36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,15,19,19,19,24,24,21,42,24,24,21,21,21,21,43,16,24,24,24,21,21,21,39,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,44,32,32,37,36,35,19,19,19,19,24,24,25,21,44,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,41,15,15,15,15,15,36,36,39,32,32,37,36,36,36,36,32,32,36,36,36,36,39,37,36,35,19,24,24,21,
		36,36,36,36,36,36,36,36,36,15,15,15,15,38,36,36,36,36,36,36,36,36,36,41,15,19,19,19,24,24,21,42,24,24,21,21,21,21,43,16,24,24,25,21,21,21,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,16,24,25,44,32,32,32,36,19,19,19,19,16,24,24,21,21,36,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,36,32,32,32,36,36,36,36,32,37,36,36,36,36,32,36,36,36,19,24,24,21,
		36,36,36,36,36,36,36,36,36,41,15,15,15,38,36,36,36,36,36,36,36,36,36,15,15,19,19,19,24,24,21,42,24,24,21,21,21,21,43,24,24,24,25,21,21,21,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,36,36,36,41,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,16,24,25,21,39,32,32,35,19,19,19,16,24,24,25,21,34,36,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,15,15,15,15,15,38,36,36,36,39,32,32,32,36,36,36,32,32,36,36,36,39,32,36,36,36,19,16,24,25,
		36,36,36,36,36,36,36,36,36,41,15,15,15,38,36,36,36,36,36,36,36,36,36,15,15,19,19,19,24,24,25,21,24,24,21,21,21,21,21,24,24,24,21,21,21,44,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,36,41,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,16,24,25,21,39,32,32,19,19,19,16,24,24,24,25,21,37,36,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,15,15,15,15,15,38,36,36,36,36,39,32,32,37,36,36,32,32,36,36,36,39,37,36,36,36,35,16,24,25,
		36,36,36,36,36,36,36,36,36,41,15,15,15,38,36,36,36,36,36,36,36,36,36,15,20,19,19,19,24,24,25,21,24,24,21,21,21,21,21,24,24,24,21,21,21,44,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,36,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,21,44,32,33,19,19,19,24,24,24,25,21,34,41,15,38,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,32,32,32,36,36,32,32,36,36,36,32,37,36,36,36,36,19,24,24,
		36,36,36,36,36,36,36,36,36,41,15,15,15,38,36,36,36,36,36,36,36,36,41,15,20,19,19,19,24,24,25,21,24,24,21,21,21,21,42,24,24,25,21,21,21,44,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,41,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,21,44,32,19,19,19,24,24,24,24,21,21,32,41,15,15,15,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,15,15,15,15,15,15,36,36,36,36,36,36,39,32,32,37,36,32,32,36,36,36,32,37,36,36,36,36,19,24,24,
		36,36,36,36,36,36,36,36,36,41,15,15,15,38,36,36,36,36,36,36,36,36,41,15,20,19,19,19,16,24,25,21,24,24,21,21,21,21,42,24,24,21,21,21,21,36,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,41,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,25,21,35,19,19,24,24,24,24,25,21,34,37,36,15,15,15,36,36,36,36,41,15,15,15,15,15,36,36,36,36,36,36,36,15,15,15,15,15,15,36,36,36,36,36,36,36,32,32,32,36,32,32,36,36,36,32,36,36,36,36,36,35,16,24,
		36,36,36,36,36,36,36,36,36,41,15,15,15,15,36,36,36,36,36,36,36,36,15,15,20,19,19,19,16,24,25,21,24,24,21,21,21,21,42,24,24,21,21,21,21,36,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,36,36,36,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,16,24,25,21,19,19,16,24,24,24,24,21,21,32,36,36,36,15,15,38,36,36,36,41,15,15,15,15,15,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,36,36,36,36,36,39,32,32,37,32,32,37,36,36,32,36,36,36,36,36,35,16,24,
		36,36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,36,15,15,20,19,19,19,16,24,25,21,42,24,21,21,21,21,42,24,24,21,21,21,21,36,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,36,36,36,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,16,24,25,21,24,19,24,24,24,24,21,21,34,37,36,36,36,41,15,38,36,36,36,15,15,15,15,15,15,36,36,36,36,36,36,41,15,15,15,15,15,38,36,36,36,36,36,36,36,36,32,32,32,32,32,37,36,39,37,36,36,36,36,36,36,19,24,
		36,36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,41,15,15,20,19,19,19,16,24,25,21,42,24,25,21,21,21,42,24,24,21,21,21,44,39,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,36,36,41,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,16,24,25,21,24,16,24,24,24,25,21,21,32,37,36,36,36,36,15,15,36,36,36,15,15,15,15,15,15,36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,39,32,32,32,32,37,36,39,37,36,36,36,36,36,36,19,16,
		36,36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,41,15,15,20,19,19,19,16,24,25,21,42,24,25,21,21,21,42,24,24,21,21,21,44,39,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,16,24,25,21,24,24,24,24,25,21,21,21,37,36,36,36,36,36,41,15,38,36,41,15,15,15,15,15,15,36,36,36,36,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,32,32,32,32,37,36,39,36,36,36,36,36,36,36,35,19,
		36,36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,15,15,15,20,19,19,19,16,24,24,21,42,24,25,21,21,21,42,24,21,21,21,21,44,39,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,16,24,25,21,24,24,24,24,21,21,21,44,36,36,36,36,36,36,41,15,15,38,41,15,15,15,15,15,15,15,36,36,36,41,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,39,32,32,32,37,36,32,36,36,36,36,36,36,36,35,19,
		36,36,36,36,36,36,36,36,36,36,15,15,15,15,36,36,36,36,36,36,36,15,15,15,20,19,19,19,16,24,24,21,21,24,25,21,21,21,42,24,21,21,21,21,36,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,16,24,25,21,24,24,24,25,21,21,21,44,36,36,36,36,36,36,36,15,15,15,41,15,15,15,15,15,15,15,36,36,36,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,32,32,32,32,39,32,36,36,36,36,36,36,36,36,19,
		36,36,36,36,36,36,36,36,36,36,15,15,15,15,38,36,36,36,36,36,41,15,15,15,36,19,19,19,16,24,24,21,21,24,24,21,21,21,42,25,21,21,21,21,39,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,38,36,41,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,21,24,24,24,21,21,21,21,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,36,36,41,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,39,32,36,36,36,36,36,36,36,36,19,
		36,36,36,36,36,36,36,36,36,36,41,15,15,15,38,36,36,36,36,36,15,15,15,38,36,19,19,19,16,24,24,21,21,24,24,21,21,21,42,25,21,21,21,44,32,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,38,36,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,42,24,24,21,21,21,21,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,41,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,36,36,36,36,36,36,36,36,19,
		36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,36,36,36,36,36,15,15,15,36,36,19,19,19,16,24,24,21,21,42,24,25,21,21,42,25,21,21,21,34,32,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,38,36,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,42,24,24,21,21,21,21,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,36,36,36,39,36,36,36,36,35,
		36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,36,36,36,36,15,15,15,15,36,36,35,19,19,16,24,24,25,21,42,24,25,21,21,42,25,21,21,21,34,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,38,41,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,21,21,24,24,21,21,21,44,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,32,32,32,32,32,36,36,36,32,36,36,36,36,35,
		36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,38,36,36,41,15,15,15,38,36,36,35,19,19,16,24,24,25,21,21,24,25,21,21,42,24,21,21,21,34,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,38,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,25,21,42,24,21,21,21,44,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,32,32,32,32,32,36,36,39,32,32,32,32,36,35,
		36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,38,36,36,15,15,15,15,36,36,36,35,19,19,19,24,24,25,21,21,24,25,21,21,21,24,21,21,21,34,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,38,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,24,21,42,24,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,32,32,32,32,32,36,36,39,32,32,32,32,36,36,
		36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,38,36,41,15,15,15,38,36,36,36,35,19,19,19,24,24,24,21,21,24,25,21,21,21,24,21,21,21,34,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,24,24,24,21,42,24,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,39,32,32,32,32,32,36,36,
		36,36,36,36,36,36,36,37,36,36,36,15,15,15,15,15,36,15,15,15,15,36,36,36,36,35,19,19,19,24,24,24,21,21,24,25,21,21,21,24,21,21,21,34,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,16,24,24,21,42,24,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,36,
		36,36,36,36,36,36,36,37,36,36,36,15,15,15,15,15,41,15,15,15,15,36,36,36,36,35,19,19,19,24,24,24,25,21,42,24,21,21,21,24,21,21,21,44,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,21,21,24,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,37,36,36,36,15,15,15,15,15,15,15,15,15,38,36,36,36,36,35,19,19,19,24,24,24,25,21,42,25,21,21,21,24,21,21,21,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,21,21,24,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,36,15,15,15,15,15,15,15,15,15,36,36,36,36,36,35,19,19,19,24,24,24,25,21,42,25,21,21,21,24,21,21,21,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,25,21,24,21,21,21,44,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,41,15,15,15,15,15,15,15,15,15,36,36,36,36,36,35,19,19,19,24,24,24,24,21,42,25,21,21,21,24,21,21,21,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,24,24,25,21,24,21,21,21,44,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,32,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,41,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,21,36,36,36,39,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,25,21,42,25,21,21,44,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,32,32,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,41,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,21,36,36,36,39,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,21,42,25,21,21,21,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,41,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,21,36,36,36,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,21,42,25,21,21,21,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,32,36,36,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,37,36,36,41,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,21,36,36,36,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,25,42,24,21,21,21,44,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,32,32,36,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,37,36,36,41,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,21,36,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,38,36,36,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,25,42,24,21,21,21,44,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,39,32,32,36,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,37,36,36,41,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,44,36,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,38,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,25,21,24,21,21,21,21,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,39,32,32,37,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,37,36,36,36,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,44,36,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,24,21,24,21,21,21,21,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,39,32,32,32,32,37,36,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,36,15,15,15,15,15,15,15,15,36,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,44,36,36,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,24,21,24,25,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,32,32,32,32,32,32,37,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,36,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,44,36,36,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,39,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,16,24,24,24,25,24,25,21,21,21,44,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,36,36,36,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,36,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,24,25,42,25,21,21,21,21,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,38,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,37,36,39,29,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,36,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,36,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,24,25,42,24,21,21,21,21,44,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,39,32,37,36,39,29,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,37,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,39,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,35,19,19,19,24,24,24,25,21,24,21,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,15,36,36,36,36,36,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,32,32,37,36,32,41,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,32,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,39,32,37,36,37,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,19,19,19,19,24,24,24,24,21,24,21,21,21,21,44,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,36,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,36,32,32,37,39,32,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,32,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,36,32,32,37,36,37,36,36,36,36,39,32,37,36,36,36,36,36,36,36,36,36,36,36,19,19,19,19,24,24,24,24,21,42,25,21,21,21,21,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,36,39,32,37,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,39,32,32,37,39,32,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,32,32,32,36,36,36,36,36,36,36,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,39,32,32,36,36,37,36,36,36,36,39,32,32,37,36,36,36,36,36,36,36,36,36,36,19,19,19,19,16,24,24,24,21,42,25,21,21,21,21,36,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,39,32,32,32,32,37,36,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,16,24,24,24,21,42,25,21,21,21,24,21,21,34,32,32,32,37,36,36,36,36,36,36,36,36,36,36,36,36,36,39,15,15,15,15,15,15,15,39,32,32,36,39,37,36,36,36,36,39,32,32,32,37,36,36,36,36,36,36,36,36,36,19,19,19,19,16,24,24,24,25,42,24,21,21,21,21,44,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,39,32,32,32,32,37,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,24,24,24,24,21,42,25,21,21,42,24,21,21,34,32,32,32,32,36,36,39,37,36,36,36,36,36,36,36,36,39,32,15,15,15,15,15,15,15,32,32,32,36,32,37,36,36,36,36,39,32,32,32,32,36,36,36,36,36,36,36,36,36,19,19,19,19,16,24,24,24,25,21,24,21,21,21,21,44,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,15,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		36,36,36,36,36,39,32,32,32,32,37,15,15,15,15,15,15,15,15,38,36,36,36,36,36,36,19,19,19,24,24,24,24,21,42,25,21,21,42,24,21,21,34,32,32,32,32,32,36,32,32,36,36,36,36,36,36,36,39,32,32,15,15,15,15,15,15,15,32,32,32,39,32,37,36,36,36,36,32,32,32,32,32,32,37,36,36,36,36,36,36,32,19,19,19,19,16,24,24,24,25,21,24,25,21,21,21,44,36,36,36,36,36,36,36,36,41,15,15,15,15,15,15,15,38,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,36,36,36,36,32,32,32,32,32,37,15,15,15,15,15,15,15,15,38,36,36,36,36,36,35,19,19,19,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,37,36,36,36,36,36,39,32,32,32,15,15,15,15,15,15,31,32,32,32,32,32,37,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,19,19,16,24,24,24,25,21,24,25,21,21,21,21,36,36,36,36,36,36,36,36,15,15,15,15,15,15,15,15,38,36,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,38,36,36,36,36,36,35,19,19,19,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,36,36,36,36,32,32,32,32,15,15,15,15,15,15,31,32,32,32,32,32,37,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,19,19,16,24,24,24,24,21,24,24,21,21,21,21,32,37,36,36,36,36,36,36,15,15,15,15,15,15,15,15,38,36,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,36,36,36,36,36,35,19,19,19,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,36,36,39,32,32,32,29,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,19,19,16,24,24,24,24,21,24,24,21,21,21,21,34,37,36,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,36,36,36,36,36,35,19,19,16,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,39,32,32,32,32,29,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,19,19,16,24,19,24,24,21,42,24,25,21,21,21,34,32,37,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,39,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,36,36,36,36,36,35,19,19,16,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,19,19,16,24,19,16,24,21,42,24,25,21,21,21,21,32,32,36,36,36,36,41,15,15,15,15,15,15,15,15,38,36,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,38,36,36,36,36,35,19,19,16,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,19,19,19,19,16,24,19,16,24,25,21,24,24,21,21,21,21,32,32,32,36,36,36,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,31,32,37,36,36,35,19,19,16,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,31,32,32,32,32,19,19,19,19,16,24,19,16,24,25,21,42,24,21,21,21,21,34,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,32,32,37,36,35,19,19,24,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,31,32,32,32,32,19,19,19,19,16,24,19,19,24,25,21,42,24,25,21,21,21,34,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,32,32,32,36,19,19,19,24,24,24,24,24,21,42,25,21,21,42,25,21,21,34,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,31,32,32,32,32,32,29,31,32,32,32,33,19,19,19,19,24,24,19,19,24,25,21,21,24,25,21,21,21,34,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,32,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,19,19,19,24,24,24,24,24,21,42,25,21,21,42,25,21,21,21,32,32,29,31,32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,31,32,32,32,32,32,15,31,32,32,32,33,19,19,19,19,24,24,45,19,24,24,21,42,24,25,21,21,21,21,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,31,32,32,32,32,32,32,32,32,15,15,15,32,32,32,32,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,32,32,32,33,19,19,16,24,24,24,24,24,21,42,25,21,21,42,24,21,21,21,32,32,15,31,32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,29,15,31,32,32,32,32,15,31,32,32,32,33,19,19,19,19,24,24,45,19,24,24,21,42,24,24,21,21,21,21,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,31,32,32,32,29,32,32,29,15,15,15,15,32,32,32,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,31,32,32,33,19,19,16,24,24,16,24,24,21,42,25,21,21,21,24,21,21,21,32,32,15,31,32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,29,15,31,32,32,32,29,15,31,32,32,32,20,19,19,19,19,24,24,45,19,24,24,21,21,24,24,21,21,21,21,34,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,31,32,29,15,31,29,15,15,15,15,15,31,32,32,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,15,32,32,33,19,19,16,24,45,16,24,24,21,42,25,21,21,21,24,21,21,21,31,15,15,15,32,32,32,32,32,29,15,15,15,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,29,15,31,32,32,32,29,15,15,32,32,29,20,19,19,19,19,24,24,45,19,16,24,25,21,24,24,25,21,21,21,34,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,32,15,15,15,15,15,15,15,15,15,15,15,15,
		32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,15,15,15,31,32,19,19,19,24,24,19,16,24,24,21,42,25,21,21,21,24,21,21,21,15,15,15,15,31,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,29,15,15,32,32,32,15,15,15,31,32,15,19,19,19,19,19,24,24,45,19,16,24,25,21,42,22,23,21,21,21,34,32,32,32,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
		32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,31,32,19,19,19,24,24,19,16,24,24,21,42,24,21,21,21,24,25,21,21,15,15,15,15,15,31,32,32,32,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,29,15,15,31,32,32,15,15,15,31,29,15,19,19,19,19,19,24,24,45,19,16,24,25,26,13,9,46,21,21,21,21,32,32,32,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
		32,32,29,15,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,31,33,19,19,19,24,45,19,16,24,24,21,42,24,21,21,21,24,25,21,21,15,15,15,15,15,15,31,32,29,15,15,15,15,15,15,15,15,15,47,9,15,15,15,47,15,15,15,15,31,32,32,32,32,32,32,32,29,15,15,31,32,29,15,15,15,15,29,15,19,19,19,19,16,24,24,45,19,16,24,25,13,13,9,24,21,21,21,21,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
		32,32,15,15,15,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,33,19,19,16,24,45,19,24,24,24,21,21,24,21,21,21,42,25,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,9,48,15,15,15,9,15,15,15,9,15,15,15,31,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,19,19,19,19,24,24,24,45,19,16,24,13,13,12,9,24,21,21,21,21,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
		32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,19,16,24,45,19,24,24,24,21,21,24,21,21,21,42,25,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,9,48,15,15,13,9,15,15,47,9,15,15,15,15,15,31,32,32,15,15,15,15,15,15,15,48,15,15,15,20,19,19,19,19,24,24,24,24,19,16,22,13,13,9,49,24,21,21,25,21,32,32,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,
		32,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,19,16,24,45,19,24,24,24,21,21,24,21,21,21,42,25,21,21,30,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,13,9,9,15,14,13,9,15,15,12,9,15,15,15,15,15,15,15,15,15,15,15,15,15,14,12,9,9,9,48,20,19,19,19,16,24,24,24,45,19,16,13,13,12,9,24,24,21,21,25,21,32,32,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,13,
		32,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,20,19,19,19,16,24,19,19,24,24,24,21,21,42,21,21,21,42,25,21,21,21,15,15,15,15,15,15,15,15,15,15,15,15,15,47,48,14,13,9,9,13,13,13,9,15,13,12,9,48,15,14,48,15,15,15,15,15,15,15,15,15,15,15,13,13,12,9,20,19,19,19,16,24,24,24,45,19,22,13,13,12,9,24,24,21,21,24,21,32,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,15,15,15,15,15,15,14,9,15,15,15,15,14,13,
		15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,19,19,19,19,24,24,19,19,24,24,24,21,21,42,25,21,21,42,24,21,21,21,15,15,15,15,15,15,15,15,15,15,15,15,13,9,48,11,13,13,13,13,13,13,9,11,13,12,9,48,15,13,9,15,15,15,15,15,15,15,15,15,15,15,14,13,13,9,50,19,19,19,16,24,24,24,45,19,22,13,13,9,46,24,24,21,21,24,21,34,29,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,47,48,15,15,15,15,14,9,15,15,15,15,14,13,
		15,15,15,15,15,15,15,15,15,15,15,13,9,9,9,9,9,15,15,15,15,15,19,19,19,19,24,24,19,19,24,24,24,21,21,42,25,21,21,21,24,21,21,21,30,15,15,15,15,15,15,15,15,15,9,14,13,9,11,13,13,13,13,13,13,13,12,11,13,13,9,9,13,13,12,9,48,15,15,15,15,15,15,15,15,15,15,13,13,12,9,19,19,19,16,24,24,24,19,19,13,13,12,9,46,24,24,21,21,24,21,30,15,15,15,15,48,15,15,15,15,15,15,15,15,15,15,15,15,32,32,32,32,32,32,32,32,32,32,32,32,32,32,15,15,15,9,15,15,15,15,15,15,12,9,48,15,15,13,13,
		15,15,15,15,15,15,15,15,15,15,14,13,13,13,13,12,9,48,15,15,15,20,19,19,19,19,24,24,19,19,24,24,24,21,21,24,25,21,21,21,24,21,21,21,30,15,15,15,15,15,15,15,15,14,9,14,13,9,11,13,13,13,13,13,13,13,13,13,13,13,13,9,13,13,12,9,48,14,12,48,15,15,15,15,15,15,15,15,13,13,9,50,19,19,24,24,24,24,19,19,13,13,12,9,21,24,24,21,21,24,21,30,15,15,15,9,9,15,15,15,15,15,15,15,15,15,15,15,15,31,32,32,32,32,29,15,15,31,32,32,32,32,29,15,14,12,48,15,15,15,15,15,15,12,9,9,15,15,13,13,
		15,15,15,15,15,15,15,15,15,15,14,15,15,15,15,13,13,9,48,15,15,19,19,19,19,16,24,24,19,19,24,24,24,21,12,9,24,21,21,21,24,25,21,21,21,15,15,15,15,15,15,15,15,14,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,13,13,12,48,15,15,15,15,15,15,15,15,13,13,9,9,19,19,24,24,24,24,19,27,13,13,9,9,21,24,24,25,21,24,21,30,15,15,13,9,48,15,15,15,15,15,15,15,15,15,15,15,15,15,32,29,15,51,15,15,15,15,32,15,15,32,15,14,13,9,48,15,15,15,15,15,15,14,12,9,9,15,13,13,
		15,15,15,12,9,15,15,15,15,15,15,15,15,15,15,14,13,12,9,15,15,19,19,19,19,16,24,24,19,19,24,24,22,13,9,9,24,21,21,21,24,25,21,21,21,15,15,15,15,15,15,47,15,14,13,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,11,13,13,13,9,15,15,15,15,15,15,15,15,14,13,12,9,19,19,24,24,24,24,19,27,13,13,9,46,21,24,24,25,21,24,21,21,15,13,12,9,15,15,15,15,15,15,15,15,15,15,15,15,15,15,31,15,15,9,9,9,15,15,15,15,15,15,14,13,13,9,15,15,15,15,15,15,15,14,13,9,9,9,11,13,
		15,15,14,12,48,15,15,15,15,15,15,15,15,15,15,15,13,13,9,9,15,19,19,19,19,16,24,24,19,16,24,22,13,12,46,24,24,21,21,21,26,13,9,21,21,15,15,15,15,15,15,47,15,14,13,12,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,15,15,15,15,47,15,15,15,14,13,12,9,9,19,24,24,24,45,19,13,13,13,9,46,21,24,24,25,21,24,21,21,14,13,9,48,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,9,9,48,15,15,15,15,15,14,13,12,9,15,15,15,15,15,15,15,15,13,9,9,9,11,13,
		15,15,14,9,15,15,15,15,15,15,15,15,15,15,15,15,14,13,12,9,50,19,19,19,19,16,24,24,19,16,24,13,13,9,21,42,24,21,21,21,13,12,9,21,21,15,15,15,15,15,15,47,48,14,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,15,15,15,14,9,9,48,15,14,13,13,12,9,19,24,24,24,45,19,13,13,13,9,46,21,24,24,25,21,24,25,21,13,13,9,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,47,9,9,15,15,15,15,15,13,13,9,48,15,15,15,15,15,15,15,15,13,12,9,9,11,13,
		15,15,13,9,15,15,15,15,15,15,15,15,15,15,15,15,15,13,13,9,9,19,19,19,19,24,24,24,19,16,13,13,12,46,21,42,24,25,21,26,13,9,21,21,21,30,15,15,15,15,15,47,48,14,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,15,15,15,14,13,9,48,15,14,13,13,12,9,19,24,24,24,45,19,13,13,13,9,46,21,24,24,25,21,42,25,26,13,12,9,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,9,9,48,15,15,15,14,13,12,9,48,15,15,15,15,15,15,15,15,13,13,9,9,13,13,
		15,15,13,9,15,15,15,15,15,15,12,9,9,48,15,15,15,13,13,12,9,50,19,19,19,24,24,24,19,27,13,13,12,46,21,21,24,25,21,13,12,49,21,21,21,30,15,15,15,15,15,12,9,14,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,48,14,9,15,13,13,9,15,14,13,13,13,9,50,24,24,24,19,19,13,13,13,9,46,21,24,24,25,21,42,25,26,13,9,9,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,12,9,9,15,15,15,13,13,12,9,15,15,15,15,15,15,15,15,15,13,13,13,11,13,13,
		15,14,13,9,15,15,15,15,15,15,14,13,13,9,48,15,15,14,13,13,9,9,19,19,19,24,24,24,19,13,13,13,9,46,21,21,24,25,26,13,12,49,21,21,21,30,15,15,15,15,15,13,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,11,13,9,48,13,13,9,15,14,13,13,13,9,50,24,24,24,19,19,13,13,13,9,9,21,24,24,25,21,42,25,13,13,9,9,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,14,9,9,15,15,14,13,13,12,9,15,15,15,15,15,15,15,15,15,13,13,13,13,13,13,
		15,14,13,9,15,15,15,15,15,15,15,14,13,13,9,15,15,14,13,13,9,9,19,19,19,24,24,24,27,13,13,12,9,25,21,21,24,25,26,13,9,24,25,21,21,30,15,15,15,15,15,13,12,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,13,13,12,15,14,13,13,13,12,9,24,24,45,19,19,13,13,13,9,9,46,24,24,24,21,42,22,13,12,9,48,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,14,12,9,48,15,13,13,13,12,48,15,15,15,15,15,15,15,15,14,13,13,13,13,13,13,
		15,14,13,9,15,15,15,15,15,15,15,15,13,13,12,48,15,14,13,13,12,9,50,19,16,24,24,45,13,13,13,12,9,25,21,21,24,24,13,12,9,24,25,21,21,21,15,15,15,15,15,13,13,12,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,13,13,12,48,13,13,13,13,13,9,49,24,45,19,19,13,13,13,9,9,9,24,24,24,21,42,22,13,12,9,48,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,14,12,9,9,15,13,13,13,9,48,15,15,15,15,15,15,15,15,14,13,13,13,13,13,13,
		15,14,13,9,15,15,15,15,15,15,15,15,13,13,13,9,15,14,13,13,13,9,9,19,16,24,24,45,13,13,13,9,9,25,21,21,24,24,13,12,9,42,25,21,21,21,15,15,15,15,15,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,13,13,12,48,13,13,13,13,13,12,9,24,45,19,27,13,13,13,9,9,9,24,24,24,21,21,22,13,12,9,48,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,15,14,13,9,9,15,13,13,13,9,48,15,15,15,15,15,15,15,15,14,13,13,13,13,13,13,
		15,13,13,9,15,15,15,9,9,15,15,15,14,13,13,12,48,14,13,13,13,9,9,50,16,24,24,45,13,13,13,9,9,25,21,21,42,24,13,12,9,42,24,21,21,21,15,15,15,15,15,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,13,13,13,13,13,12,9,9,50,52,9,13,13,13,12,9,9,49,24,24,25,21,13,13,12,9,48,15,15,15,15,15,15,15,15,47,9,9,15,15,15,15,15,15,15,15,15,14,13,9,9,48,13,13,13,9,15,15,15,15,15,15,12,15,15,13,13,13,13,13,13,13,
		14,13,13,9,15,15,15,13,12,48,15,15,14,13,13,13,9,14,13,13,13,12,9,9,16,24,24,22,13,13,13,9,9,25,21,21,42,22,13,12,9,49,24,21,21,21,15,15,15,47,15,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,13,13,13,13,13,13,9,9,9,9,9,13,13,13,12,9,9,9,24,24,25,26,13,13,12,9,48,15,15,15,15,15,15,15,15,15,9,9,48,15,15,15,15,15,15,15,15,14,13,12,9,11,13,13,12,9,15,15,15,15,15,14,12,48,47,13,13,13,13,13,13,13,
		14,13,13,9,15,15,15,14,13,9,15,15,14,13,13,13,9,14,13,13,13,13,9,9,49,24,24,45,13,13,13,9,9,46,21,21,42,22,13,12,9,46,24,21,21,21,30,15,15,47,48,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,11,13,13,13,13,9,9,9,9,24,25,13,13,13,12,9,48,15,15,15,15,15,15,15,15,15,13,9,48,15,15,15,15,15,15,15,15,14,13,13,9,11,13,13,12,9,48,15,15,15,15,14,12,48,9,13,13,13,13,13,13,13,
		13,13,13,9,48,15,15,14,13,12,48,15,13,13,13,13,12,11,13,13,13,13,13,9,9,24,24,22,13,13,13,9,9,9,21,21,53,13,13,12,9,9,24,21,21,21,30,15,15,13,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,13,13,13,13,13,9,9,9,9,49,22,13,13,13,12,9,48,15,15,15,15,15,15,15,15,15,13,9,9,15,15,15,15,15,15,15,15,14,13,13,12,13,13,13,12,9,48,15,15,15,15,14,13,9,11,13,13,13,13,13,13,13,
		13,13,13,12,48,15,15,13,13,12,9,11,13,13,13,13,12,11,13,13,13,13,13,12,9,49,24,13,13,13,13,9,9,9,9,11,13,13,13,13,9,9,49,54,9,21,30,15,14,13,12,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,9,13,13,13,13,12,9,9,15,15,15,15,15,15,15,15,15,13,13,9,48,15,15,15,15,15,9,15,13,13,13,13,13,13,13,12,9,48,15,15,15,15,13,13,13,13,13,13,13,13,13,13,13,
		13,13,13,13,9,15,14,13,13,13,9,13,13,13,13,13,13,9,13,13,13,13,13,13,52,9,54,13,13,13,13,12,9,9,9,13,13,13,13,13,12,9,9,9,9,9,30,9,11,13,13,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,9,9,11,13,13,13,12,9,9,15,15,15,15,15,15,15,15,15,13,13,12,9,15,15,15,15,13,9,15,13,13,13,13,13,13,13,12,9,9,15,15,15,15,13,13,13,13,13,13,13,13,13,13,13,
		13,13,13,13,9,48,13,13,13,13,13,13,13,13,13,13,13,12,13,13,13,13,13,13,13,12,9,13,13,13,13,13,13,9,11,13,13,13,13,13,13,12,9,9,9,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,13,13,13,13,13,9,9,48,15,15,15,15,15,15,15,14,13,13,12,9,15,15,15,15,13,48,14,13,13,13,13,13,13,13,12,9,9,48,15,15,14,13,13,13,13,13,13,13,13,13,13,13,
		13,13,13,13,12,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,13,13,13,13,13,13,9,9,48,15,15,15,15,15,15,15,14,13,13,13,9,48,15,47,15,13,48,11,13,13,13,13,13,13,13,13,9,9,9,15,15,13,13,13,13,13,13,13,13,13,13,13,13,
		13,13,13,13,13,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,15,15,15,15,15,15,14,13,13,13,13,12,9,9,9,13,13,9,11,13,13,13,13,13,13,13,13,12,9,9,48,14,13,13,13,13,13,13,13,7,3,56,13,13,
		13,55,56,13,13,13,13,13,13,13,13,13,13,13,13,13,55,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,6,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,48,15,15,11,13,13,13,13,13,13,13,13,13,9,11,13,13,9,13,13,13,13,13,13,13,13,13,13,9,9,9,13,13,13,13,13,13,13,55,7,3,56,13,13,
		13,55,56,13,13,13,13,13,57,56,13,13,13,13,13,55,6,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,7,7,3,56,13,13,13,13,13,13,56,13,13,13,13,13,13,13,55,6,3,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,57,3,13,13,13,13,13,55,6,3,56,13,13,13,13,13,13,13,13,13,13,12,9,9,9,9,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,56,13,13,13,13,12,9,9,13,13,13,13,13,13,13,7,7,3,56,13,13,
		13,55,3,13,13,13,13,13,57,56,13,13,13,13,13,7,6,56,13,13,13,13,6,56,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,57,13,13,13,13,13,55,7,7,3,13,13,13,13,13,13,13,3,13,13,13,13,13,13,13,13,13,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,6,3,13,13,13,55,7,7,3,3,13,13,13,13,13,13,13,13,13,13,13,12,9,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,6,56,13,13,13,13,13,12,11,13,13,13,13,13,13,55,7,7,6,3,13,13,
		13,55,3,13,13,13,13,13,55,3,13,13,13,13,55,7,6,13,13,13,13,13,55,3,13,13,13,13,13,13,13,13,7,6,56,13,13,13,13,13,13,3,3,13,13,13,13,13,55,7,7,3,13,13,13,13,13,55,7,3,13,13,13,13,13,13,13,13,13,7,7,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,7,3,56,13,13,55,7,7,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,12,9,9,11,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,7,7,3,3,13,
		13,55,6,56,13,13,13,13,55,6,56,13,13,13,55,7,3,13,13,13,13,13,55,6,56,13,13,13,13,13,13,7,7,3,13,13,13,13,13,13,55,3,56,13,13,13,13,13,7,7,6,56,13,13,13,13,13,7,7,3,13,13,13,13,13,13,13,13,13,55,7,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,3,56,13,13,7,7,7,6,3,56,13,13,13,13,13,13,13,3,56,13,13,13,13,12,9,13,13,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,7,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,7,7,6,3,3,
		13,55,6,56,13,13,13,13,7,6,3,13,13,13,55,7,3,13,13,13,13,13,55,7,3,13,13,13,13,13,7,7,6,56,13,13,13,13,13,13,7,3,56,13,13,13,13,13,7,7,6,56,13,13,13,13,13,7,7,3,13,13,13,13,13,13,13,13,13,55,7,6,56,13,13,13,13,13,13,13,13,13,57,13,13,13,13,13,13,13,13,55,7,3,13,13,55,7,7,7,6,3,13,13,13,13,13,13,13,13,6,3,56,13,13,13,13,13,13,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,7,7,7,7,7,3,3,
		13,55,7,3,13,13,13,13,7,7,3,13,13,13,7,7,3,13,13,13,13,13,55,7,3,13,13,13,13,55,7,7,6,56,13,13,13,13,13,55,7,3,13,13,13,13,13,13,7,7,6,56,13,13,13,13,55,7,6,3,13,13,13,13,13,13,13,13,13,55,7,7,56,13,13,13,13,13,13,13,13,13,3,13,13,13,13,13,13,13,13,7,7,3,56,13,7,7,7,7,6,3,13,13,13,13,13,13,13,13,7,6,3,13,13,13,13,13,57,3,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,55,7,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,7,7,7,7,7,7,6,
		13,7,7,3,56,13,13,55,7,7,3,56,13,13,7,7,3,56,13,13,13,13,55,7,6,56,13,13,13,7,7,7,6,56,13,13,13,13,13,55,7,3,13,13,13,13,13,13,7,7,6,3,13,13,13,13,55,7,6,3,13,13,13,13,13,13,13,13,13,7,7,7,3,13,13,13,13,13,13,13,13,57,3,13,13,13,13,13,13,13,13,7,7,6,3,13,7,7,7,7,6,3,13,13,13,13,13,13,13,55,7,7,3,56,13,13,13,55,6,3,13,13,13,13,13,13,13,13,13,3,13,13,13,13,13,55,7,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,7,7,7,7,7,7,7,
		13,7,7,6,56,13,13,7,7,7,6,3,13,13,7,7,3,56,13,13,13,13,55,7,6,56,13,13,13,7,7,7,6,3,13,13,13,13,13,55,7,3,13,13,13,13,13,13,7,7,6,3,13,13,13,13,55,7,6,3,56,13,13,13,13,13,13,13,13,7,7,7,3,13,13,13,13,13,13,13,13,6,3,13,13,13,13,13,13,13,13,7,7,6,3,56,7,7,7,7,6,3,13,13,13,13,13,13,13,55,7,7,6,56,13,13,13,7,6,3,13,13,13,13,13,13,13,13,13,3,56,13,13,13,13,7,7,7,3,56,13,13,13,13,13,13,13,13,13,13,13,13,13,7,7,7,7,7,7,7,
		7,7,7,7,3,56,55,7,7,7,6,3,56,13,7,7,3,3,13,13,13,13,7,7,7,3,13,13,55,7,7,7,6,3,13,13,55,56,13,55,7,3,56,13,13,13,13,55,7,7,7,3,13,13,13,13,55,7,7,3,56,13,13,13,13,13,13,13,13,7,7,7,6,56,13,13,13,13,13,13,13,6,3,13,13,13,13,13,13,13,13,7,7,7,3,3,7,7,7,7,6,3,13,13,13,13,13,13,13,7,7,7,6,56,13,13,7,7,6,3,13,13,13,13,13,13,13,13,13,6,3,13,13,13,13,7,7,7,6,3,13,13,13,13,13,13,13,13,6,13,13,13,13,7,7,7,7,7,7,7,
		7,7,7,7,6,3,7,7,7,7,6,3,3,3,7,7,3,3,56,13,13,13,7,7,7,3,56,13,55,7,7,7,6,3,3,5,7,56,13,55,7,3,3,13,13,13,13,7,7,7,7,3,56,13,13,13,7,7,7,6,3,13,13,13,13,13,13,13,55,7,7,7,6,3,13,13,13,13,13,13,13,6,3,56,13,13,13,13,13,13,55,7,7,7,7,7,7,7,7,7,7,3,56,13,13,13,13,13,55,7,7,7,6,56,13,13,7,7,6,3,56,13,13,13,13,13,13,13,13,7,3,56,13,13,13,7,7,7,7,3,3,13,13,13,13,13,13,55,6,13,13,13,13,7,7,7,7,7,7,7,
		7,7,7,7,7,6,7,7,7,7,7,3,3,5,7,7,6,3,3,13,13,55,7,7,7,3,3,13,55,7,7,7,7,3,3,5,7,56,13,55,7,6,3,56,13,13,55,7,7,7,7,6,3,58,13,13,7,7,7,6,3,56,13,13,13,13,13,13,7,7,7,7,7,3,3,13,13,13,13,13,55,6,3,3,13,13,13,13,13,13,55,7,7,7,7,7,7,7,7,7,7,3,3,13,13,13,13,13,7,7,7,7,6,56,13,13,7,7,6,3,3,13,13,13,13,13,13,13,55,7,6,56,13,13,13,7,7,7,7,6,3,3,13,13,13,13,13,7,6,56,13,13,13,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,3,7,7,7,7,3,3,7,7,7,7,7,7,6,3,56,7,7,7,7,7,7,7,7,7,56,13,55,7,6,3,3,3,3,5,7,7,7,7,7,3,5,13,55,7,7,7,7,3,3,13,13,13,13,13,55,7,7,7,7,7,7,3,3,13,13,13,13,7,6,3,3,56,13,13,13,13,13,55,7,7,7,7,7,7,7,7,7,7,6,3,13,13,13,13,55,7,7,7,7,6,56,13,13,7,7,6,3,3,56,13,13,13,13,13,13,7,7,6,3,13,13,13,7,7,7,7,7,3,3,3,56,13,13,7,7,6,3,56,13,13,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,7,7,7,7,7,7,7,7,7,3,56,7,7,7,3,3,3,3,5,7,7,7,7,7,7,3,5,7,7,7,7,7,6,3,7,13,13,13,55,7,7,7,7,7,7,7,6,3,3,7,13,7,7,7,3,3,3,56,13,13,13,55,7,7,7,7,7,7,7,7,7,7,7,7,3,3,13,13,13,7,7,7,7,7,6,5,13,55,7,7,6,3,3,3,13,13,13,13,13,55,7,7,7,3,13,13,55,7,7,7,7,7,7,3,3,3,5,7,7,7,7,3,3,13,13,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,7,7,7,7,6,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,7,7,7,7,7,7,7,7,7,7,7,7,3,7,7,7,7,7,3,3,3,3,3,3,55,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,7,7,7,7,7,7,7,7,3,5,7,7,7,7,3,3,3,56,13,13,13,7,7,7,7,7,6,56,13,55,7,7,7,7,7,7,7,3,3,7,7,7,7,7,7,3,56,55,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,3,3,3,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,5,7,7,7,7,3,3,3,3,5,7,7,7,7,7,7,6,3,56,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,3,5,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,6,3,3,3,7,7,7,7,7,7,7,7,6,3,5,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7
	};
	int color_palette[] = {
		3675682320,2517669392,2517660876,1959556300,1959564816,1959552010,1678406860,1678402570,1678398280,1397248840,1397253130,1397242502,981881672,981875334,2083469958,2083486767,1382574856,981892143,981881448,1382568552,2083476072,2206892938,1795701382,981894026,1795713800,1795720074,2206874246,1382562438,981887752,2230615087,2206891055,2083489012,2230617332,2230604392,2206893300,2927317608,2927341179,2230627963,2083499643,2927330548,2083482376,2927328303,2206886664,2206880360,2206903931,1795707496,1397261194,2083476296,1397259311,1397254920,1397248616,2230604616,1382568776,2206880584,1795707720,981885962,1959541382,981890252,1678391942
	};
	char* bitmap = compressed_bitmap;
	int x, y, *VRAM = GetVRAMAddress();
	VRAM += 16;
	for(y=216 ; y ; y--, VRAM+=32) {
		for(x=160 ; x ; x--)
			*(VRAM++) = color_palette[(int)*(bitmap++)];
	}
}

void draw_tipsPanel()
{
	const char bmp[] = {
		0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,
		1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,1,
		1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,2,1,
		1,3,4,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,4,2,1,
		1,3,4,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,4,2,1,
		1,3,4,2,1,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,5,1,3,4,2,1,
		1,3,4,2,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,3,4,2,1,
		1,3,4,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,4,2,1,
		1,3,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,2,1,
		1,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,1,
		0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0
	};
	const short palette[] = {65535,12546,50220,20868,33382,20933,31368};
	CopySpriteMasked(bmp, palette, (SCREEN_WIDTH-224)/2, 1, 224, 48, MASK);
}
